import java.awt.*;
import freaklamarsch.awt.*;
import java.awt.event.*;
import java.util.*;

/**
* <code>Dialog</code> to set the Compiler Options for <code>Javacw</code>
* @see Javacw
* @author FReAK La Marsch
*/
public class CompileOptions extends Dialog
{
    private Checkbox gAll, optimize, nowarn, verbose, deprecation, classpath, directory, autoadd, depend, javaoption;
    private TextField gSomeInput, classpathInput, directoryInput, javaoptionInput;
    private Button setButton, cancelButton, saveButton, classpathInputButton, directoryInputButton;
    private Properties p;
    private Javacw owner;

    private ActionListener setAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            updateSettings();
            owner.setAppConfig(p);
            dispose();
        }
    };

    private ActionListener showAutobuiltAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            updateSettings();
            owner.setAppConfig(p);
            dispose();
        }
    };

    private ActionListener cancelAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            dispose();
        }
    };

    private ActionListener saveAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            updateSettings();
            owner.setAppConfig(p);
            try
            {
                owner.saveProject("CompilerOptions");
            }
            catch(Exception e)
            {
                System.out.println("Couldn't save Options because:\n"+e);
            }
            dispose();
        }
    };

    class InputButtonActionListener implements ActionListener
    {
        private TextField inputField;

        InputButtonActionListener(TextField tf)
        {
            inputField = tf;
        }

        public void actionPerformed(ActionEvent evt)
        {
            FLMFileDialog fd = new FLMFileDialog(owner, System.getProperty("user.dir"), "Add path...");
            fd.setVisible(true);
            String inputLabel = inputField.getName();
            String inputValue = inputField.getText();
            String dir = fd.getDirectory();
            if (inputLabel.equals("classpath"))
            {
                String file = fd.getFile();
                try
                {
                    if (file.toUpperCase().endsWith(".JAR") || file.toUpperCase().endsWith(".ZIP"))
                    {
                        inputValue = (inputValue.equals("")) ? dir+file : inputValue+";"+dir+file;
                    }
                    else
                    {
                        inputValue = (inputValue.equals("")) ? dir : inputValue+";"+dir;
                    }
                }
                catch(NullPointerException npe)
                {
                    if(dir != null)
                    {
                        inputValue = (inputValue.equals("")) ? dir : inputValue+";"+dir;//this is thrown, if no file is selected in the dialog
                    }
                }
            }
            else if (inputLabel.equals("directory"))
            {
                inputValue = dir;
            }
            if(inputValue!=null)
            {
                inputField.setText(inputValue);
            }
            fd.dispose();
            System.gc();//the fileDialog trashes the iPAQ somehow 
        }
    };

    CompileOptions(Javacw ow)
    {
        super(ow, "Compiler options", true);
        this.owner = ow;
        p = owner.getAppConfig();

        //Listener to close this Dialog without sending data
        addWindowListener(new WindowAdapter()
            {
            public void windowClosing(WindowEvent e)
            {
                dispose();
            }
        });

        //Add the input elements to this Dialog
        setFont(new Font("Arial", Font.PLAIN, 10));

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(gridbag);

        gAll = (p.getProperty("javacw.compilerOptions.g.switch").equals("on")) ? new Checkbox("-g", /*gGroup,*/ true) : new Checkbox("-g", /*gGroup,*/ false);
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(gAll, constraints);

        optimize = new Checkbox("-O");
        optimize.setState(p.getProperty("javacw.compilerOptions.optimize.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 16;
        constraints.weightx = 0;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(optimize, constraints);

        nowarn = new Checkbox("-nowarn");
        nowarn.setState(p.getProperty("javacw.compilerOptions.nowarn.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 16;
        constraints.weightx = 0;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(nowarn, constraints);

        verbose = new Checkbox("-verbose");
        verbose.setState(p.getProperty("javacw.compilerOptions.verbose.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.NONE;
        add(verbose, constraints);

        deprecation = new Checkbox("-deprecation");
        deprecation.setState(p.getProperty("javacw.compilerOptions.deprecation.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 5;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.NONE;
        add(deprecation, constraints);

        classpath = new Checkbox("-classpath");
        classpath.setState(p.getProperty("javacw.compilerOptions.classpath.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 6;
        constraints.gridwidth = 7;
        constraints.weightx = 1;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classpath, constraints);

        classpathInput = new TextField(p.getProperty("javacw.compilerOptions.classpath.input"),10);
        classpathInput.setName("classpath");
        constraints.gridx = 7;
        constraints.gridy = 6;
        constraints.gridwidth = 7;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classpathInput, constraints);

        classpathInputButton = new Button("...");
        classpathInputButton.addActionListener(new InputButtonActionListener(classpathInput));
        constraints.gridx = 14;
        constraints.gridy = 6;
        constraints.gridwidth = 2;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.NONE;
        add(classpathInputButton, constraints);

        autoadd = new Checkbox("Autobuild classpath");
        autoadd.setState(p.getProperty("javacw.compilerOptions.autoaddPath.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 7;
        constraints.gridwidth = 12;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(autoadd, constraints);

        /*classpathInputButton = new Button("view");
        classpathInputButton.addActionListener(showAutobuiltAction);
        constraints.gridx = 13;
        constraints.gridy = 7;
        constraints.gridwidth = 3;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.NONE;
        add(classpathInputButton, constraints);*/

        directory = new Checkbox("-d");
        directory.setState(p.getProperty("javacw.compilerOptions.directory.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 8;
        constraints.gridwidth = 7;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(directory, constraints);

        directoryInput = new TextField(p.getProperty("javacw.compilerOptions.directory.input"),10);
        directoryInput.setName("directory");
        constraints.gridx = 7;
        constraints.gridy = 8;
        constraints.gridwidth = 7;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(directoryInput, constraints);

        directoryInputButton = new Button("...");
        directoryInputButton.addActionListener(new InputButtonActionListener(directoryInput));
        constraints.gridx = 14;
        constraints.gridy = 8;
        constraints.gridwidth = 2;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.NONE;
        add(directoryInputButton, constraints);

        depend = new Checkbox("-depend");
        depend.setState(p.getProperty("javacw.compilerOptions.depend.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 9;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(depend, constraints);

        javaoption = new Checkbox("-J");
        javaoption.setState(p.getProperty("javacw.compilerOptions.j.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 10;
        constraints.gridwidth = 7;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(javaoption, constraints);

        javaoptionInput = new TextField(p.getProperty("javacw.compilerOptions.j.input"),10);
        javaoptionInput.setName("javaoption");
        constraints.gridx = 7;
        constraints.gridy = 10;
        constraints.gridwidth = 7;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(javaoptionInput, constraints);

        setButton = new Button("set");
        setButton.addActionListener(setAction);
        constraints.gridx = 1;
        constraints.gridy = 11;
        constraints.gridwidth = 4;
        constraints.insets = new Insets(5,2,0,2);
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.weightx = 0.33;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(setButton, constraints);

        cancelButton = new Button("cancel");
        cancelButton.addActionListener(cancelAction);
        constraints.gridx = 5;
        constraints.gridy = 11;
        constraints.gridwidth = 4;
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.weightx = 0.33;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(cancelButton, constraints);

        saveButton = new Button("save");
        saveButton.addActionListener(saveAction);
        constraints.gridx = 9;
        constraints.gridy = 11;
        constraints.gridwidth = 4;
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.weightx = 0.33;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(saveButton, constraints);

        validate();
        pack();
    }

    private void updateSettings()
    {
        p.put("javacw.compilerOptions.g.switch", (gAll.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.optimize.switch", (optimize.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.nowarn.switch", (nowarn.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.verbose.switch", (verbose.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.deprecation.switch", (deprecation.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.classpath.switch", (classpath.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.classpath.input", classpathInput.getText());
        p.put("javacw.compilerOptions.autoaddPath.switch",(autoadd.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.directory.switch", (directory.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.directory.input", directoryInput.getText());
        p.put("javacw.compilerOptions.depend.switch", (depend.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.j.switch", (javaoption.getState()) ? "on" : "off");
        p.put("javacw.compilerOptions.j.input", javaoptionInput.getText());
    }
}
