import java.awt.*;
import freaklamarsch.awt.*;
import java.awt.event.*;
import java.io.*;
import freaklamarsch.io.*;
import java.util.*;
import freaklamarsch.util.*;
import java.lang.*;
import freaklamarsch.tools.*;

/**
* This wrapper makes it possible to compile java programs on platforms that don't have a command line,
* but a java runtime.<br>
* On systems with a command line it can be used as GUI, but this wrapper was intentionally written
* for running on Compaqs iPAQ, so the graphics are optimized to display correct on this system (240 x 320).<br>
* When the wrapper is started you'll see a <code>MenuBar</code> where ever it's displayed by your system, some
* <code>Button</code>s at top of the <code>Frame</code> for quick access to some functions, a <code>TextArea</code>
* for the system output in the center of the screen and a status bar at the bottom of the <code>Frame</code>
* which provides information about what's going on.<br>
* <br>
* The <code>MenuBar</code>:<br>
* <br>
* <table>
* <tr>
* <td colspan="3"><b>File</b></td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>New</b>: Removes all files from the compiler</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Open Project</b>: Opens an existing Javac Wrapper Project (.jwp)</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Save Project</b>: Saves the current project</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Save Project as ...</b>: Allows you to save the current project into another file</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Add File</b>: Opens a <code>FLMFileDialog</code> where you can select the file that should be added to the compiler.</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Add Directory</b>: Opens a <code>FLMFileDialog</code> where you can select a file or directory that should be added to the compiler.<br>
* If you choose a file, this file will be added as first file to the compiler and after that all other .java-files will be added to the compiler<br>
* If you choose a directory all .java-files from this directory will be added to the compiler. The first file to be added is the first .java-file in the directory.</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Remove file(s)</b>: Opens a <code>Dialog</code> where you can choose the files to be removed from the compiler.</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Compile</b>: Calls sun.tools.javac.Main.compile(String[]) with all added files and the options selected in the options menu</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Exit</b></td>
* </tr>
* <tr>
* <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
* </tr>
* <tr>
* <td colspan="3"><b>Options</b></td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Compiler Options</b>: Opens a <code>Dialog</code> where you can select the options that should be used when the files are compiled</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Run Options</b>: Opens a <code>Dialog</code> where you can select the options that should be used when the compiled application is started</td>
* </tr>
* </table>
* <br>
* <table>
* <tr>
* <td valign="top" colspan="3">The Compiler Options:</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>g</b>: Generate all debugging info</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>O</b>: Optimize; may hinder debugging or enlarge class file</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>nowarn</b>: Generate no warnings</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>verbose</b>: Output messages about what the compiler is doing</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>deprecation</b>: Output source locations where deprecated APIs are used</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>classpath &lt;path&gt;</b>: Specify where to find user class files</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Autobuild classpath</b>: If this option is enabled, the paths of the added files will be added to the classpath automatically</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>d &lt;directory&gt;</b>: Specify where to place generated class files</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>depend</b>: Recompilation of class files on which the source files given as arguments recursively depend</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>J&lt;javaoption&gt;</b>: Passes through the string javaoption as a single argument to the Java interpreter which runs the compiler. The argument should not contain spaces. Multiple argument words must all begin with the prefix -J, to the first argument -J is added automatically</td>
* </tr>
* </table>
* <br>
* <table>
* <tr>
* <td valign="top" colspan="3">The Run Options:</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Allow run by invoke</b>: If this is enabled, the runButton will be enabled although the System doesn't support the Runtime.exec() command.<br>
* In this case, the application is started by invoking it's main method from within the wrapper. This sounds easy, but it causes some substantial problems which should be solved in this release, but I can't guarantee that there're no bugs. One still existing problem is that if your compiled application calls <code>System.exit(int)</code> it will cause the wrapper to close too. This problem is solved for Java 1.2, so if you want to run the wrapper with a Java 1.2 runtime take the sources, remove the comments in SecurityManager part in Javacw.java and recompile it.<br>
* (For iPAQ Users: The JVM for the iPAQ has a bug which causes the JVM to crash, whenever you hide a frame that was created from your application.)
* </td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Autobuild classpath</b>: If this option is enabled, the paths of the added files will be added to the classpath automatically. This may cause some problems if you are trying to run a class which is part of a package</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>classpath &lt;path&gt;</b>: Specify where to find user class files</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Class name to run</b>: Enter a class file which should be executed when the runButton is pressed</td>
* </tr>
* </table>
* <br>
* <table>
* <tr>
* <td valign="top" colspan="3">The <code>Buttons</code>:</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Add File...</b>: Opens a <code>FileDialog</code> where you can select the file that should be added to the compiler.</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Compile</b>: Calls sun.tools.javac.main with all added files and the options selected in the Options menu</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>Run</b>: Tries to run the compiled class if it is an application with a main method</td>
* </tr>
* <tr>
* <td valign="top">&nbsp;</td><td valign="top">-</td><td><b>New</b>: Removes all files from the compiler</td>
* </tr>
* </table>
*<br>
* <b>Note!</b> The wrapper uses the java compiler found in the package sun.tools.javac which is not supported by Sun.<br>
* In newer releases (and some older ones) of the JDK <code>sun.tools.javac.Main</code> will be removed, so you have to make sure that it is present somewhere else on the system and that the wrapper has access to it.<br>
*<br>
* @author FReAK La Marsch, Yani Ioannou
*/
public class Javacw extends Frame
{
    private final Color panelcolor = Color.lightGray;
    private MenuBar menuBar;
    private Panel nBar;
    private Panel sBar;
    private Label status;
    private TextArea cText;
    private Button addButton, compileButton, runButton, newButton, optionsButton;
    private Javacw javacw = this;
    private Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    private OutputStream stdOut;
    private PrintStream stdPrint;
    private String[] cOptionsArray =  {/*0 -g*/"off", /*-O*/"off", /*-nowarn*/"off", /*-verbose*/"off", /*-deprecation*/"off", /*5 -classpath*/"on", ""+System.getProperty("java.class.path"), /*autoaddPath*/"on", /*-d*/"off", "", /*10 -depend*/"off", /*-J*/"off", ""};//Default options, length == 13
    private String[] rOptionsArray =  {/*0 allow run*/"off", /*add files*/"on", /*classpath*/"on", ""+System.getProperty("java.class.path"),/*workingdir*/ "off",/*5 classname*/ "", "off", ""};//Default options, length == 8
    private String filesClasspath = "";
    private Properties p;
    private boolean running = false;
    private boolean fileCompiled = false;
    private boolean runAllowed = false;
    private boolean invokeAllowed = false;
    private boolean autoaddPath;
    private FLMClassRunner cr;
    private FLMHashMap menuState = new FLMHashMap();
    private Properties appConfig;
    private CompilerSecurityManager sm = new CompilerSecurityManager();
    private FLMResizableArray sourceFiles = new FLMResizableArray();
    private String projectName = "Javacw.ini";
    private String projectDir = System.getProperty("user.dir");
    private String[] sourceFilesArray = new String[0];
    private boolean projectChanged = false;
    boolean overwrite = false;
    /**
    * Action for compiling the files.  
    * Calls compile(String[])
    */
    private ActionListener compileAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            printStatus("Compiling...");
            nBar.setEnabled(false);
            Component[] disableFilter = {addButton, compileButton, runButton, newButton};
            disableComponents(disableFilter);
            cText.setText("");
            System.out.println("Started...");
            String[] params = getCompileOptions();
            if (compile(params))
            {
                fileCompiled =  true;
                printStatus("File(s) successfully compiled");
            }
            else
            {
                printStatus("Compiling failed");
            }
            nBar.setEnabled(true);
            Component[] enableFilter = {addButton, compileButton, runButton, newButton};
            enableComponents(enableFilter);
            System.gc();
        }
    };

    /**
    * Generates the compiler options according to the values stored
    * in the appConfig <code>Properties</code>.
    * @author FReAK La Marsch
    * @return String[] containing all enabled options and their values 
    */
    private String[] getCompileOptions()
    {
        FLMResizableArray storage = new FLMResizableArray();

        if(appConfig.getProperty("javacw.compilerOptions.g.switch").equals("on"))
        {
            storage.add("-g");
        }
        if(appConfig.getProperty("javacw.compilerOptions.optimize.switch").equals("on"))
        {
            storage.add("-O");
        }
        if(appConfig.getProperty("javacw.compilerOptions.nowarn.switch").equals("on"))
        {
            storage.add("-nowarn");
        }
        if(appConfig.getProperty("javacw.compilerOptions.verbose.switch").equals("on"))
        {
            storage.add("-verbose");
        }
        if(appConfig.getProperty("javacw.compilerOptions.deprecation.switch").equals("on"))
        {
            storage.add("-deprecation");
        }

        if(appConfig.getProperty("javacw.compilerOptions.classpath.switch").equals("on"))
        {
            storage.add("-classpath");
            storage.add((appConfig.getProperty("javacw.compilerOptions.autoaddPath.switch").equals("on") ? filesClasspath+appConfig.getProperty("javacw.compilerOptions.classpath.input") : appConfig.getProperty("javacw.compilerOptions.classpath.input")));
        }

        if(appConfig.getProperty("javacw.compilerOptions.directory.switch").equals("on"))
        {
            storage.add("-d");
            storage.add(appConfig.getProperty("javacw.compilerOptions.directory.input"));
        }

        if(appConfig.getProperty("javacw.compilerOptions.depend.switch").equals("on"))
        {
            storage.add("-depend");
        }

        if(appConfig.getProperty("javacw.compilerOptions.j.switch").equals("on"))
        {
            storage.add("-J"+appConfig.getProperty("javacw.compilerOptions.j.input"));
        }

        int storageSize = storage.size();
		int sourceFilesSize = sourceFiles.size();
        String[] optionsArray = new String[storageSize+sourceFilesSize];
        
		for(int k=0; k<storageSize; k++)
        {
            optionsArray[k] = (String) storage.get(k);
        }

        //storageSize += sourceFilesSize;

        for(int l = 0; l<sourceFilesSize; l++)
        {
            optionsArray[storageSize + l] = (String) sourceFiles.get(l);
        }
        return optionsArray;
    }

    /*
    * Action to end the wrapper
    * @author FReAK La Marsch
    */
    private ActionListener exitAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            exit(0);
        }
    };

    /*
    * Action to remove Files from the wrapper.  
    * Opens a Dialog and waits for user input.
    * @author FReAK La Marsch
    */
    private ActionListener removeAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            RemoveFileDialog rfd = new RemoveFileDialog(javacw);
			rfd.show();
            projectChanged = true;
            printStatus("Removed selected file(s); " + sourceFiles.size() + " input files");
			rfd.dispose();
			rfd=null;
			updateFilesClasspath();
            System.gc();//the fileDialog trashes the iPAQ somehow 
        }
    };

    /**
    * addAction opens a new <code>FLMFileDialog</code> in which the user can choose a file.  <br>
    * If the sourceFiles <code>FLMResizableArray</code> doesn't contain this file already,
    * this file is added to the sourceFiles <code>FLMResizableArray</code>.
    *
    * @author FReAK La Marsch
    */
    private ActionListener addAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            FLMFileDialog fd = new FLMFileDialog(javacw, projectDir, "Select source file", new String[]{"*.java"});
			fd.setVisible(true);
            String file =  fd.getDirectory() + fd.getFile();
            if(file != null && file.toUpperCase().endsWith(".JAVA"))
            {
                if(!sourceFilesContains(file))
                {
                    sourceFiles.add(file);

                    if(filesClasspath.indexOf(fd.getDirectory())==-1)
                    {
                        filesClasspath = filesClasspath + fd.getDirectory() + ";";
                    }
                    printStatus("Added " + fd.getFile() + "; " + sourceFiles.size() + " input files");
                    newButton.setEnabled(true);
                    compileButton.setEnabled(true);
                    menuState.put("New", new Boolean(true));
                    menuState.put("Compile", new Boolean(true));
                    menuState.put("RemoveFile", new Boolean(true));
                    updateMenuBarState();
                    projectChanged = true;
                }
                else
                {
                    printStatus("File already added; " + sourceFiles.size() + " input files");
                }
            }
            else
            {
                printStatus("You didn't select a .java-File; " + sourceFiles.size() + " input files");
            }
            fd.dispose();
			fd=null;
            System.gc();//the fileDialog trashes the iPAQ somehow 
        }
    };

    /**
    * addDirAction opens a new FileDialog in which the user can choose a file.  <br>
    * If the sourceFiles <code>FLMResizableArray</code> is empty, this file is added as first file,
    * so it will be executed when the user clicks the run Button.
    * After this, all .java-Files form this directory are also added if the sourceFiles <code>FLMResizableArray</code>
    * doesn't contain this file already.
    *
    * @author FReAK La Marsch
    */
    private ActionListener addDirAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            FLMFileDialog fd = new FLMFileDialog(javacw, projectDir, "Select main file");
            FLMFilenameFilter filter = new FLMFilenameFilter("*.java");
            fd.setFilenameFilter(filter, "*.java");
            fd.setVisible(true);
            String dir = fd.getDirectory();
            String file = fd.getFile();
            //System.out.println(dir + " / "+file);
            String[] files = (dir != null) ? (new File(dir)).list(filter) : new String[0];

            if (files.length>0)
            {        
                String mainFile = (dir.endsWith(File.separator)) ? dir + fd.getFile() : dir + File.separator + fd.getFile();

                if(sourceFiles.size() == 0 && mainFile !=null && mainFile.toUpperCase().endsWith(".JAVA"))
                {
                    sourceFiles.add(mainFile);
                }
                for(int i = 0; i<files.length; i++)
                {
                    String fileString = (dir.endsWith(File.separator)) ? dir + files[i] : dir + File.separator + files[i];
                    if(!sourceFilesContains(fileString))
                    {
                        sourceFiles.add(fileString);
                        if(filesClasspath.indexOf(fd.getDirectory())==-1)
                        {
                            filesClasspath = filesClasspath + fd.getDirectory() + ";";
                        }
                        projectChanged = true;
                        printStatus("Added " + fd.getFile() + "; " + sourceFiles.size() + " input files");
                    }
                    else
                    {
                        printStatus("File already added; " + sourceFiles.size() + " input files");
                    }
                }

                printStatus("Added all files from directory; " + sourceFiles.size() + " input files");
                newButton.setEnabled(true);
                compileButton.setEnabled(true);
                menuState.put("New", new Boolean(true));
                menuState.put("Compile", new Boolean(true));
                menuState.put("RemoveFile", new Boolean(true));
                updateMenuBarState();
            }
            else
            {
                printStatus("No .java-Files in selection; " + sourceFiles.size() + " input files");
            }
            fd.dispose();
			fd=null;
            System.gc();//the fileDialog trashes the iPAQ somehow 
        }
    };

    /**
    * Method to check if a file is already added to the compiler
    * @author FReAK La Marsch
    */
    private boolean sourceFilesContains(String fileString)
    {
        for(int i=0; i<sourceFiles.size(); i++)
        {
            if(((String) sourceFiles.get(i)).equals(fileString))
            {
                return true;
            }
        }
        return false;
    }
    
    private void updateFilesClasspath()
    {
        String cp= "";
        String path;
        for(int i=0; i<sourceFiles.size(); i++)
        {
            path = (String) sourceFiles.get(i);
            path = path.substring(0, path.lastIndexOf(File.separator));
            if(cp.indexOf(path)==-1)
            {
                cp += (path+";");
            }
        }
        filesClasspath = cp.toString();
    }

    /**
    * This ActionListener tries to run the main file (first file added to the wrapper) if it is an
    * application.  <br>
    * This action is invoked by the runButton.<br>
    * The Personal Java Runtime for WindowsCE doesn't support the Runtime.exec() method.<br>
    * If you would call this method on WindowsCE systems an UnsupportedOperationException is thrown.<br>
    * The problem with this is that this method is needed for starting applications (as far as I know)
    * and that's why this feature is not available for WindowsCE devices.<br>
    * Instead of calling the Runtime.exec() method on systems that have this problem it's tried to get
    * and invoke the main method of the compiled class.<br>
    * This allows to start some small programs, but may fail on more complex application<br>
    * @author FReAK La Marsch
    */
    private ActionListener runAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            String cmd = evt.getActionCommand();
            if (cmd == "Run")
            {
                try
                {
                    cText.setText("");
                    printStatus("trying to run program");
                    String mainFile = appConfig.getProperty("javacw.runOptions.classname.switch").equals("on") ? appConfig.getProperty("javacw.runOptions.classname.input") : ((String) sourceFiles.get(0));
                    runClass(mainFile.substring(0, mainFile.length()-5) + ".class");
                }
                catch(Exception e)
                {
                    System.out.println("Error while trying to run the program: " + e);
                    printStatus("done running program");
                }
            }
            else if (cmd=="Kill")
            {
                cr.kill();                
            }
        }
    };

    /**
    * ActionListener for the <code>MenuItem</code> About in the Help <code>Menu</code>.  <br>
    * When the ActionEvent occures it prints some Info on std.out.<br>
    * @author FReAK La Marsch
    */
    private ActionListener aboutAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            cText.setText("");
			System.out.println("At this point I want to thank the both\nguys who helped to develop this\nsoftware.\n\nFirst there is Frank Bodman\n(http://www.hasiland.com/javaonepoc/javac.html),\nhe wrote an article on how to\ncompile Java on Epoc\nand second Yani Ioannou\n(http://csc.dhs.org/java.php)\nhe started to write this program\nand published his sources for others\nto develop it further.\n\nWithout these guy this\nprogram wouldn't exist.\nThanks a lot!");
        }
    };

    /**
    * ActionListener for the <code>MenuItem</code> Compiler Options in the Options <code>Menu</code>.  <br>
    * When the ActionEvent occures it creates a new <code>CompileOptions</code> object.<br>
    * @author FReAK La Marsch
    * @see CompileOptions
    */
    private ActionListener cOptionsAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            CompileOptions compileOptions = new CompileOptions(javacw);
            compileOptions.show();
        }
    };

    /**
    * ActionListener for the <code>MenuItem</code> Run Options in the Options <code>Menu</code>.  <br>
    * When the ActionEvent occures it creates a new <code>CompileOptions</code> object.<br>
    * @author FReAK La Marsch
    * @see CompileOptions
    */
    private ActionListener rOptionsAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            RunOptions runOptions = new RunOptions(javacw);
            runOptions.show();
            updateRunButtonState();
        }
    };

    private ActionListener autobuiltAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            System.out.println("Current Autobuilt Classpath:\n"+filesClasspath);
        }
    };

    /**
    * ActionListener for the <code>MenuItem</code> "New" in the File <code>Menu</code>.  <br>
    * When the ActionEvent occures the runButton, runMenuItem, newButton and newMenuItem are disabled and the <code>FLMResizableArray</code> for the input files is emptyed<br>
    * @author Yani Ioannou, FReAK La Marsch
    */
    private ActionListener newBuildAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            clear();
        }
    };

    private ActionListener openProjectAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            if (!projectChanged || askForSave())
            {
                printStatus("Opening project");
                FLMFileDialog fd = new FLMFileDialog(javacw, projectDir, "Open project ...");
                fd.setFilenameFilter(new FLMFilenameFilter("*.jwp"), "*.jwp");//.jwp = Javac Wrapper Project
                fd.setVisible(true);
                if(fd.getFile() != null && fd.getDirectory() != null)
                {
                    projectName = fd.getFile();
                    projectDir = fd.getDirectory();
                    fd.dispose();
					fd=null;
                    System.gc();//the fileDialog trashes the iPAQ somehow
                    loadConfig();
                    updateFilesClasspath();
                    menuState.put("RemoveFile", ((sourceFiles.size()>0)? new Boolean(true) : new Boolean(false)));
                    menuState.put("Compile", ((sourceFiles.size()>0)? new Boolean(true) : new Boolean(false)));
                    menuState.put("SaveProject", new Boolean(true));
                    updateMenuBarState();
                    compileButton.setEnabled((sourceFiles.size()>0)? true : false);
                    printStatus("Opened project "+projectName+", "+sourceFiles.size() + " input files");
                    newButton.setEnabled(true);
                }
                else
                {
                    printStatus("Couldn't open the project");
	                fd.dispose();
    	            fd=null;
        	        System.gc();
                }
            }
        }
    };

    private ActionListener saveProjectAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            try
            {
                printStatus("Saveing project as ...");
                saveProject("All");
                printStatus("Saved the project");
            }
            catch(Exception e)
            {
                System.out.println("Error while trying to save the Project:\n"+e);
                printStatus("Couldn't save the project");
            }

        }
    };

    private ActionListener saveProjectAsAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            printStatus("Saveing project as ...");
            FLMFileDialog fd = new FLMFileDialog(javacw, projectDir, "Save project as ...", new String[]{"*.jwp"}, FLMFileDialog.SAVE);
            //fd.setFilenameFilter(new FLMFilenameFilter("*.jwp"));//.jwp = Javac Wrapper Project
            fd.setVisible(true);
            if(fd.getFile() != null && fd.getDirectory() != null)
            {
                boolean addedJWP = false;
                if (!fd.getFile().toUpperCase().endsWith(".JWP"))
                {
                    fd.setFile(fd.getFile()+".jwp");
                    addedJWP = true;
                }
                projectName = fd.getFile();
                projectDir = fd.getDirectory();
                if (new File(fd.getDirectory()+fd.getFile()).exists())
                {
                    FLMOkCancelDialog ocd = new FLMOkCancelDialog(javacw, "Overwrite?", "The file \n"+fd.getFile()+"\nalready exists!\nDo you want to overwrite that file?", "Yes", "No", null);
                    ocd.addOkButtonListener(new ActionListener()
                        {
                        public void actionPerformed(ActionEvent evt)
                        {
                            overwrite = true;
                        }
                    });
                    ocd.show();
                }
                if (!new File(fd.getDirectory()+fd.getFile()).exists() || overwrite || !addedJWP)
                {
                    try
                    {
                        overwrite = false;
                        saveProject("All");
                        printStatus("Saved the project");
                        menuState.put("SaveProject", new Boolean(true));
                        updateMenuBarState();
                    }
                    catch(Exception e)
                    {
                        System.out.println("Error while trying to save the Project:\n"+e);
                        printStatus("Couldn't save the project");
                    }
                }
                else
                {
                    printStatus("Couldn't save project");
                }               
            }
            else
            {
                printStatus("Couldn't save project");
            }
			fd.dispose();
			fd=null;
			System.gc();
        }
    };

    /**
    * resets the wrapper
    * @author FReAK La Marsch
    */
    private void clear()
    {
        sourceFiles=new FLMResizableArray();
        cText.setText("");
        compileButton.setEnabled(false);
        runButton.setEnabled(false);
        newButton.setEnabled(true);
        menuState.put("Compile", new Boolean(false));
        menuState.put("New", new Boolean(false));
        menuState.put("RemoveFile", new Boolean(false));
        menuState.put("SaveProject", new Boolean(false));
        updateMenuBarState();
        filesClasspath = "";
        String projectName = "Javacw.ini";
        projectDir = System.getProperty("user.dir");
        sourceFilesArray = new String[0];
        projectChanged = false;
        System.gc();//clean it really
        printStatus("Removed all input Files; 0 input files");
    }

    /**
    * Main method of this application<br>
    * @author Yani Ioannou
    */
    public static void main(String args[])
    {
        if(args.length>1)
        {
            new Javacw(args);
        }
        else if(args.length==1 && !args[0].toUpperCase().endsWith(".JAVA"))
        {
            new Javacw(args[0]);
        }
        else
        {
            new Javacw();
        }
    }

    /**
    * The default constructor method<br>
    * @author Yani Ioannou
    */
    public Javacw()
    {
        //setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/javacw16x16.gif")));
        loadConfig();
        setStreams();
        initGUI();
        initMenuBar();
        System.setSecurityManager(sm);
        updateRunButtonState();
    }

    /**
    * Another constructor method<br>
    * @author FReAK La Marsch
    * @param fileNames String Array with names of files that will be put into the sourceFiles FLMResizableArray on initialisation
    */
    public Javacw(String project)
    {
        projectName = project.substring(project.lastIndexOf(File.separator)+1);
        projectDir = project.substring(0,project.lastIndexOf(File.separator)+1);
        loadConfig();
        setStreams();
        initGUI();
        initMenuBar();
        System.setSecurityManager(sm);
        updateRunButtonState();
    }

    public Javacw(String[] fileNames)
    {
        //setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/javacw16x16.gif")));
        loadConfig();
        setStreams();
        initGUI();
        initMenuBar();
        System.setSecurityManager(sm);
        for(int i=0; i<fileNames.length; i++)
        {
            sourceFiles.add(fileNames[i]);
        }
        printStatus(sourceFiles.size() + " input files");
        compileButton.setEnabled(true);
        newButton.setEnabled(true);
        updateRunButtonState();
    }

    /**
    * Creates a new <code>OutputStream</code> and sets stdOut and stdErr to write on this stream<br>
    * @author Yani Ioannou
    */
    private void setStreams()
    {
        stdOut = new OutputStream()
        {
            StringBuffer buffer = new StringBuffer();

            public void write(int i) throws IOException
            {
                buffer.append((char) i);
                if((char) i=='\n') flush();
            }

            public void flush()
            {
                print(buffer.toString());
                buffer=new StringBuffer();
            }
        };

        stdPrint = new PrintStream(stdOut);
        System.setOut(stdPrint);
        System.setErr(stdPrint);
    }


    /*private void setIcon()
    {
        Image icon = getImage("images"+File.separator+"Javacw16x16.gif");
        //System.out.println(Javacw.getClass().getName());
        //Image icon = Toolkit.getDefaultToolkit().getImage("images"+File.separator+"Javacw16x16.gif");
        setIconImage(icon); 
    }*/
    /**
    * Sets up the GUI of this application<br>
    * @author Yani Ioannou, FReAK La Marsch
    */

    private void initGUI()
    {
        this.setVisible(false);
        //URL url = getClass().getResource("images"+File.separator+"Javacw16x16.gif");
        //System.out.println("url = "+url.getFile());
        //Image icon = Toolkit.getDefaultToolkit().getImage(url);
        //setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("images"+File.separator+"Javacw16x16.gif")));
        this.setBackground(Color.white);
        this.setLayout(new BorderLayout());
        this.addWindowListener(
            new WindowAdapter()
            {
            public void windowClosing(WindowEvent evt)
            {
                exit(0);
            }
        });

        menuBar = new MenuBar();

        /*The File Menu*/
        Menu fileMenu = new Menu("File");
        fileMenu.setName("File");
        MenuItem newBuildMenuItem = new MenuItem("New");
        newBuildMenuItem.setName("New");
        newBuildMenuItem.addActionListener(newBuildAction);
        MenuItem openProjectMenuItem = new MenuItem("Open Project");
        openProjectMenuItem.setName("OpenProject");
        openProjectMenuItem.addActionListener(openProjectAction);
        MenuItem saveProjectMenuItem = new MenuItem("Save Project");
        saveProjectMenuItem.setName("SaveProject");
        saveProjectMenuItem.addActionListener(saveProjectAction);
        MenuItem saveProjectAsMenuItem = new MenuItem("Save Project as ...");
        saveProjectAsMenuItem.setName("SaveProjectAs");
        saveProjectAsMenuItem.addActionListener(saveProjectAsAction);
        MenuItem addFileMenuItem = new MenuItem("Add File");
        addFileMenuItem.setName("AddFile");
        addFileMenuItem.addActionListener(addAction);
        MenuItem addDirMenuItem = new MenuItem("Add Directory");
        addDirMenuItem.setName("AddDir");
        addDirMenuItem.addActionListener(addDirAction);
        MenuItem removeFileMenuItem = new MenuItem("Remove File(s)");
        removeFileMenuItem.setName("RemoveFile");
        removeFileMenuItem.addActionListener(removeAction);
        MenuItem compileMenuItem = new MenuItem("Compile");
        compileMenuItem.setName("Compile");
        compileMenuItem.addActionListener(compileAction);
        MenuItem exitMenuItem = new MenuItem("Exit");
        exitMenuItem.setName("Exit");
        exitMenuItem.addActionListener(exitAction);
        fileMenu.add(newBuildMenuItem);
        fileMenu.add(openProjectMenuItem);
        fileMenu.add(saveProjectMenuItem);
        fileMenu.add(saveProjectAsMenuItem);
        fileMenu.add(addFileMenuItem);
        fileMenu.add(addDirMenuItem);
        fileMenu.add(removeFileMenuItem);
        fileMenu.add(compileMenuItem);
        fileMenu.add(exitMenuItem);

        /*The Options Menu*/
        Menu optionsMenu = new Menu("Options");
        optionsMenu.setName("Options");

        MenuItem spacerMenuItem = new MenuItem("");
	    spacerMenuItem.setName("");
	
        MenuItem cOptionsMenuItem = new MenuItem("Compiler Options");
        cOptionsMenuItem.setName("cOptions");
        cOptionsMenuItem.addActionListener(cOptionsAction);

        MenuItem rOptionsMenuItem = new MenuItem("Run Options");
        rOptionsMenuItem.setName("rOptions");
        rOptionsMenuItem.addActionListener(rOptionsAction);

        MenuItem autobuiltMenuItem = new MenuItem("Displ. Autobuilt Classpath");
        autobuiltMenuItem.setName("autobuilt");
        autobuiltMenuItem.addActionListener(autobuiltAction);

        optionsMenu.add(cOptionsMenuItem);
        optionsMenu.add(spacerMenuItem);
        optionsMenu.add(rOptionsMenuItem);
        optionsMenu.add(autobuiltMenuItem);

		/* The Help Menu */
        Menu helpMenu = new Menu("Help");
        helpMenu.setName("Help");

        MenuItem aboutMenuItem = new MenuItem("About");
        aboutMenuItem.setName("About");
        aboutMenuItem.addActionListener(aboutAction);
		
		helpMenu.add(aboutMenuItem);
        /*Add the Menus to the MenuBar*/
        menuBar.add(fileMenu);
        menuBar.add(optionsMenu);
        menuBar.add(helpMenu);

        /*Add all to the frame*/
        this.setMenuBar(menuBar);
        this.add(nBar = new Panel(), BorderLayout.NORTH);
        this.add(cText = new TextArea(),BorderLayout.CENTER);
        this.add(sBar=new Panel(), BorderLayout.SOUTH);

        /*Setup the components*/
        nBar.setLayout(new FlowLayout());
        nBar.setBackground(panelcolor);

        sBar.setLayout(new BorderLayout());
        sBar.setBackground(panelcolor);

        cText.setEditable(false);

        status = new Label();
        status.setBackground(panelcolor);
        addButton = new Button("Add File");
        addButton.addActionListener(addAction);

        compileButton = new Button("Compile");
        compileButton.setEnabled((sourceFiles.size()>0)? true : false);
        compileButton.addActionListener(compileAction);

        runButton = new Button("Run");
        runButton.setEnabled(false);
        runButton.addActionListener(runAction);

        newButton = new Button("New");
        newButton.setEnabled(true);
        //newButton.setVisible(true);
        newButton.addActionListener(newBuildAction);

        nBar.add(addButton);
        nBar.add(compileButton);
        nBar.add(runButton);
        sBar.add(status,BorderLayout.CENTER);
        nBar.add(newButton);
        this.setSize(screen.width, screen.height);

        this.setTitle("FLM Javac Wrapper");
        this.setVisible(true);
        printStatus(sourceFiles.size()+" input files");
    }


    /**
    * Sets the values in menuState and calls updateMenuState()
    * @author FReAK La Marsch
    */
    private void initMenuBar()
    {
        menuState.put("File", new Boolean(true));
        menuState.put("New", new Boolean(true));
        menuState.put("OpenProject", new Boolean(true));
        menuState.put("SaveProject", ((projectName.toUpperCase().endsWith(".JWP"))? new Boolean(true) : new Boolean(false)));
        menuState.put("SaveProjectAs", new Boolean(true));
        menuState.put("AddFile", new Boolean(true));
        menuState.put("AddDir", new Boolean(true));
        menuState.put("RemoveFile", ((sourceFiles.size()>0)? new Boolean(true) : new Boolean(false)));
        menuState.put("Compile", ((sourceFiles.size()>0)? new Boolean(true) : new Boolean(false)));
        menuState.put("Exit", new Boolean(true));
        menuState.put("Options", new Boolean(true));
        menuState.put("AllowRun", new Boolean(true));
        menuState.put("AutoAdd", new Boolean(true));
        menuState.put("cOptions", new Boolean(true));
        menuState.put("", new Boolean(false));
        menuState.put("rOptions", new Boolean(true));
        menuState.put("autobuilt", new Boolean(true));
        menuState.put("Help", new Boolean(true));
        menuState.put("About", new Boolean(true));
        updateMenuBarState();
    }

    /**
    * Enables all items of the MenuBar according to their value stored in menuState
    * @author FReAK La Marsch
    */
    private void updateMenuBarState()
    {
        for(int j=0; j<menuBar.getMenuCount(); j++)
        {
            Menu cMenu = menuBar.getMenu(j);
            cMenu.setEnabled(((Boolean)menuState.get(cMenu.getName())).booleanValue());
            for(int k=0; k<cMenu.getItemCount(); k++)
            {
                cMenu.getItem(k).setEnabled(((Boolean)menuState.get(cMenu.getItem(k).getName())).booleanValue());
            }
        }
    }

    /**
    * Sets the state of the runButton according to the state of the wrapper
    */
    private void updateRunButtonState()
    {
        runButton.setEnabled((fileCompiled || appConfig.getProperty("javacw.runOptions.classname.switch").equals("on")) && (FLMSystemUtil.canExec() || appConfig.getProperty("javacw.runOptions.invokeAllowed.switch").equals("on")));
    }

    /**
    * loadConfig tries to load the <code>Properties</code> defined in the file Javacw.ini which should be in the same directory as this application.  <br>
    * If the file can be loaded the <code>Properties</code> are written into the cOptionsArray which contains the current configuration of this program.<br>
    * If it fails, nothing is done, so that the default values for the cOptionsArray will be used as current configuration.<br>
    * @author FReAK La Marsch
    */
    private void loadConfig()
    {
        appConfig = new Properties();
        try
        {
            FileInputStream propFile = (projectDir.endsWith(File.separator)) ? new FileInputStream(projectDir+projectName) : new FileInputStream(projectDir+File.separator+projectName);
            appConfig.load(propFile);
        }
        catch (Exception e)
        { 
            //in this case the default Arrays are used
        }

        if (appConfig.getProperty("javacw.sourceFiles")!= null)
        {
            sourceFilesArray = FLMStringUtil.split(appConfig.getProperty("javacw.sourceFiles"), ";");
            for(int i = 0; i< sourceFilesArray.length; i++)
            {
                sourceFiles.add(sourceFilesArray[i]);
            }
        }

        if (appConfig.getProperty("javacw.compilerOptions.g.switch")!= null)
        {
            cOptionsArray[0] = appConfig.getProperty("javacw.compilerOptions.g.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.g.switch", cOptionsArray[0]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.optimize.switch")!=null)
        {
            cOptionsArray[1] = appConfig.getProperty("javacw.compilerOptions.optimize.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.optimize.switch", cOptionsArray[1]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.nowarn.switch")!=null)
        {
            cOptionsArray[2] = appConfig.getProperty("javacw.compilerOptions.nowarn.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.nowarn.switch", cOptionsArray[2]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.verbose.switch")!=null)
        {
            cOptionsArray[3] = appConfig.getProperty("javacw.compilerOptions.verbose.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.verbose.switch", cOptionsArray[3]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.deprecation.switch")!=null)
        {
            cOptionsArray[4] = appConfig.getProperty("javacw.compilerOptions.deprecation.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.deprecation.switch", cOptionsArray[4]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.classpath.switch")!=null)
        {
            cOptionsArray[5] = appConfig.getProperty("javacw.compilerOptions.classpath.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.classpath.switch", cOptionsArray[5]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.classpath.input")!=null)
        {
            cOptionsArray[6] = appConfig.getProperty("javacw.compilerOptions.classpath.input");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.classpath.input", cOptionsArray[6]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.autoaddPath.switch")!=null)
        {
            cOptionsArray[7] = appConfig.getProperty("javacw.compilerOptions.autoaddPath.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.autoaddPath.switch", cOptionsArray[7]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.directory.switch")!=null)
        {
            cOptionsArray[8] = appConfig.getProperty("javacw.compilerOptions.directory.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.directory.switch", cOptionsArray[8]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.directory.input")!=null)
        {
            cOptionsArray[9] = appConfig.getProperty("javacw.compilerOptions.directory.input");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.directory.input", cOptionsArray[9]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.depend.switch")!=null)
        {
            cOptionsArray[10] = appConfig.getProperty("javacw.compilerOptions.depend.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.depend.switch", cOptionsArray[10]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.j.switch")!=null)
        {
            cOptionsArray[11] = appConfig.getProperty("javacw.compilerOptions.j.switch");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.j.switch", cOptionsArray[11]);
        }

        if (appConfig.getProperty("javacw.compilerOptions.j.input")!=null)
        {
            cOptionsArray[12] = appConfig.getProperty("javacw.compilerOptions.j.input");
        }
        else
        {
            appConfig.put("javacw.compilerOptions.j.input", cOptionsArray[12]);
        }

        if (appConfig.getProperty("javacw.runOptions.invokeAllowed.switch")!=null)
        {
            rOptionsArray[0] = appConfig.getProperty("javacw.runOptions.invokeAllowed.switch");
        }
        else
        {
            appConfig.put("javacw.runOptions.invokeAllowed.switch", rOptionsArray[0]);
        }

        if (appConfig.getProperty("javacw.runOptions.autoaddPath.switch")!=null)
        {
            rOptionsArray[1] = appConfig.getProperty("javacw.runOptions.autoaddPath.switch");
        }
        else
        {
            appConfig.put("javacw.runOptions.autoaddPath.switch", rOptionsArray[1]);
        }

        if (appConfig.getProperty("javacw.runOptions.classpath.switch")!=null)
        {
            rOptionsArray[2] = appConfig.getProperty("javacw.runOptions.classpath.switch");
        }
        else
        {
            appConfig.put("javacw.runOptions.classpath.switch", rOptionsArray[2]);
        }

        if (appConfig.getProperty("javacw.runOptions.classpath.input")!=null)
        {
            rOptionsArray[3] = appConfig.getProperty("javacw.runOptions.classpath.input");
        }
        else
        {
            appConfig.put("javacw.runOptions.classpath.input", rOptionsArray[3]);
        }

        if (appConfig.getProperty("javacw.runOptions.workingdir.switch")!=null)
        {
            rOptionsArray[4] = appConfig.getProperty("javacw.runOptions.workingdir.switch");
        }
        else
        {
            appConfig.put("javacw.runOptions.workingdir.switch", rOptionsArray[4]);
        }

        if (appConfig.getProperty("javacw.runOptions.workingdir.input")!=null)
        {
            rOptionsArray[5] = appConfig.getProperty("javacw.runOptions.workingdir.input");
        }
        else
        {
            appConfig.put("javacw.runOptions.workingdir.input", rOptionsArray[5]);
        }

        if (appConfig.getProperty("javacw.runOptions.classname.switch")!=null)
        {
            rOptionsArray[6] = appConfig.getProperty("javacw.runOptions.classname.switch");
        }
        else
        {
            appConfig.put("javacw.runOptions.classname.switch", rOptionsArray[6]);
        }

        if (appConfig.getProperty("javacw.runOptions.classname.input")!=null)
        {
            rOptionsArray[7] = appConfig.getProperty("javacw.runOptions.classname.input");
        }
        else
        {
            appConfig.put("javacw.runOptions.classname.input", rOptionsArray[7]);
        }
        invokeAllowed = (appConfig.getProperty("javacw.runOptions.invokeAllowed.switch").equals("on")) ? true : false;
        runAllowed = FLMSystemUtil.canExec() ? true : invokeAllowed;
        projectChanged = false;
    }

    /**
    * Returns appConfig
    */
    protected Properties getAppConfig()
    {
        return appConfig;
    }

    /**
    * Sets appConfig
    */
    protected void setAppConfig(Properties p)
    {
        appConfig = p;
        projectChanged = true;
    }

    protected void saveProject(String type) throws FileNotFoundException, IOException
    {
        FileOutputStream out = (projectDir.endsWith(File.separator)) ? new FileOutputStream(projectDir+projectName) : new FileOutputStream(projectDir+File.separator+projectName);
        Properties applicationProps = new Properties();

        if(type.equals("CompilerOptions") || type.equals("All"))
        {
            //save the current compiler settings and update the local backup
            cOptionsArray[0] = appConfig.getProperty("javacw.compilerOptions.g.switch");
            applicationProps.put("javacw.compilerOptions.g.switch", cOptionsArray[0]);
            cOptionsArray[1] = appConfig.getProperty("javacw.compilerOptions.optimize.switch");
            applicationProps.put("javacw.compilerOptions.optimize.switch", cOptionsArray[1]);
            cOptionsArray[2] = appConfig.getProperty("javacw.compilerOptions.nowarn.switch");
            applicationProps.put("javacw.compilerOptions.nowarn.switch", cOptionsArray[2]);
            cOptionsArray[3] = appConfig.getProperty("javacw.compilerOptions.verbose.switch");
            applicationProps.put("javacw.compilerOptions.verbose.switch", cOptionsArray[3]);
            cOptionsArray[4] = appConfig.getProperty("javacw.compilerOptions.deprecation.switch");
            applicationProps.put("javacw.compilerOptions.deprecation.switch",cOptionsArray[4]);
            cOptionsArray[5] = appConfig.getProperty("javacw.compilerOptions.classpath.switch");
            applicationProps.put("javacw.compilerOptions.classpath.switch", cOptionsArray[5]);
            cOptionsArray[6] = appConfig.getProperty("javacw.compilerOptions.classpath.input");
            applicationProps.put("javacw.compilerOptions.classpath.input", cOptionsArray[6]);
            cOptionsArray[7] = appConfig.getProperty("javacw.compilerOptions.autoaddPath.switch");
            applicationProps.put("javacw.compilerOptions.autoaddPath.switch", cOptionsArray[7]);
            cOptionsArray[8] = appConfig.getProperty("javacw.compilerOptions.directory.switch");
            applicationProps.put("javacw.compilerOptions.directory.switch", cOptionsArray[8]);
            cOptionsArray[9] = appConfig.getProperty("javacw.compilerOptions.directory.input");
            applicationProps.put("javacw.compilerOptions.directory.input", cOptionsArray[9]);
            cOptionsArray[10] = appConfig.getProperty("javacw.compilerOptions.depend.switch");
            applicationProps.put("javacw.compilerOptions.depend.switch", cOptionsArray[10]);
            cOptionsArray[11] = appConfig.getProperty("javacw.compilerOptions.j.switch");
            applicationProps.put("javacw.compilerOptions.j.switch", cOptionsArray[11]);
            cOptionsArray[12] = appConfig.getProperty("javacw.compilerOptions.j.input");
            applicationProps.put("javacw.compilerOptions.j.input", cOptionsArray[12]);
            //save the run settings without changes
            applicationProps.put("javacw.runOptions.invokeAllowed.switch", rOptionsArray[0]);
            applicationProps.put("javacw.runOptions.autoaddPath.switch", rOptionsArray[1]);
            applicationProps.put("javacw.runOptions.classpath.switch", rOptionsArray[2]);
            applicationProps.put("javacw.runOptions.classpath.input", rOptionsArray[3]);
            applicationProps.put("javacw.runOptions.workingdir.switch", rOptionsArray[4]);
            applicationProps.put("javacw.runOptions.workingdir.input", rOptionsArray[5]);
            applicationProps.put("javacw.runOptions.classname.switch", rOptionsArray[6]);
            applicationProps.put("javacw.runOptions.classname.input", rOptionsArray[7]);
        }

        if(type.equals("RunOptions") || type.equals("All"))
        {
            applicationProps.put("javacw.compilerOptions.g.switch", cOptionsArray[0]);
            applicationProps.put("javacw.compilerOptions.optimize.switch", cOptionsArray[1]);
            applicationProps.put("javacw.compilerOptions.nowarn.switch", cOptionsArray[2]);
            applicationProps.put("javacw.compilerOptions.verbose.switch", cOptionsArray[3]);
            applicationProps.put("javacw.compilerOptions.deprecation.switch",cOptionsArray[4]);
            applicationProps.put("javacw.compilerOptions.classpath.switch", cOptionsArray[5]);
            applicationProps.put("javacw.compilerOptions.classpath.input", cOptionsArray[6]);
            applicationProps.put("javacw.compilerOptions.autoaddPath.switch", cOptionsArray[7]);
            applicationProps.put("javacw.compilerOptions.directory.switch", cOptionsArray[8]);
            applicationProps.put("javacw.compilerOptions.directory.input", cOptionsArray[9]);
            applicationProps.put("javacw.compilerOptions.depend.switch", cOptionsArray[10]);
            applicationProps.put("javacw.compilerOptions.j.switch", cOptionsArray[11]);
            applicationProps.put("javacw.compilerOptions.j.input", cOptionsArray[12]);
            //save the current run settings and update the local backup
            rOptionsArray[0] = appConfig.getProperty("javacw.runOptions.invokeAllowed.switch");
            applicationProps.put("javacw.runOptions.invokeAllowed.switch", rOptionsArray[0]);
            rOptionsArray[1] = appConfig.getProperty("javacw.runOptions.autoaddPath.switch");
            applicationProps.put("javacw.runOptions.autoaddPath.switch", rOptionsArray[1]);
            rOptionsArray[2] = appConfig.getProperty("javacw.runOptions.classpath.switch");
            applicationProps.put("javacw.runOptions.classpath.switch", rOptionsArray[2]);
            rOptionsArray[3] = appConfig.getProperty("javacw.runOptions.classpath.input");
            applicationProps.put("javacw.runOptions.classpath.input", rOptionsArray[3]);
            rOptionsArray[4] = appConfig.getProperty("javacw.runOptions.workingdir.switch");
            applicationProps.put("javacw.runOptions.workingdir.switch", rOptionsArray[4]);
            rOptionsArray[5] = appConfig.getProperty("javacw.runOptions.workingdir.input");
            applicationProps.put("javacw.runOptions.workingdir.input", rOptionsArray[5]);
            rOptionsArray[6] = appConfig.getProperty("javacw.runOptions.classname.switch");
            applicationProps.put("javacw.runOptions.classname.switch", rOptionsArray[6]);
            rOptionsArray[7] = appConfig.getProperty("javacw.runOptions.classname.input");
            applicationProps.put("javacw.runOptions.classname.input", rOptionsArray[7]);
        }

        if(type.equals("CompilerOptions") || type.equals("RunOptions"))
        {
            String sourceFilesString = "";
            for(int i = 0; i<sourceFilesArray.length; i++)
            {
                sourceFilesString += sourceFilesArray[i]+";";
            }
            applicationProps.put("javacw.sourceFiles", sourceFilesString);
        }
        else if(type.equals("All"))
        {
            String sourceFilesString = "";
			int size = sourceFiles.size();
            for(int i = 0; i<size; i++)
            {
                sourceFilesString += ((String)sourceFiles.get(i))+";";
            }
            applicationProps.put("javacw.sourceFiles", sourceFilesString);
        }
        applicationProps.save(out, "---!! Don't mess with .ini-files !!---");
        out.close();
        projectChanged = false;
    }
    /**
    * Saves the run options to the file Javacw.ini
    
    protected void writeRunConfig()
    { 
        try
        {
            printStatus("Saving settings to file "+projectName+" ...");
            FileOutputStream out = new FileOutputStream(projectDir+projectName);
            Properties applicationProps = new Properties();
            //save the compiler setting without changes
            applicationProps.put("javacw.compilerOptions.g.switch", cOptionsArray[0]);
            applicationProps.put("javacw.compilerOptions.optimize.switch", cOptionsArray[1]);
            applicationProps.put("javacw.compilerOptions.nowarn.switch", cOptionsArray[2]);
            applicationProps.put("javacw.compilerOptions.verbose.switch", cOptionsArray[3]);
            applicationProps.put("javacw.compilerOptions.deprecation.switch",cOptionsArray[4]);
            applicationProps.put("javacw.compilerOptions.classpath.switch", cOptionsArray[5]);
            applicationProps.put("javacw.compilerOptions.classpath.input", cOptionsArray[6]);
            applicationProps.put("javacw.compilerOptions.autoaddPath.switch", cOptionsArray[7]);
            applicationProps.put("javacw.compilerOptions.directory.switch", cOptionsArray[8]);
            applicationProps.put("javacw.compilerOptions.directory.input", cOptionsArray[9]);
            applicationProps.put("javacw.compilerOptions.depend.switch", cOptionsArray[10]);
            applicationProps.put("javacw.compilerOptions.j.switch", cOptionsArray[11]);
            applicationProps.put("javacw.compilerOptions.j.input", cOptionsArray[12]);
            //save the current run settings and update the local backup
            rOptionsArray[0] = appConfig.getProperty("javacw.runOptions.invokeAllowed.switch");
            applicationProps.put("javacw.runOptions.invokeAllowed.switch", rOptionsArray[0]);
            rOptionsArray[1] = appConfig.getProperty("javacw.runOptions.autoaddPath.switch");
            applicationProps.put("javacw.runOptions.autoaddPath.switch", rOptionsArray[1]);
            rOptionsArray[2] = appConfig.getProperty("javacw.runOptions.classpath.switch");
            applicationProps.put("javacw.runOptions.classpath.switch", rOptionsArray[2]);
            rOptionsArray[3] = appConfig.getProperty("javacw.runOptions.classpath.input");
            applicationProps.put("javacw.runOptions.classpath.input", rOptionsArray[3]);
            rOptionsArray[4] = appConfig.getProperty("javacw.runOptions.workingdir.switch");
            applicationProps.put("javacw.runOptions.workingdir.switch", rOptionsArray[4]);
            rOptionsArray[5] = appConfig.getProperty("javacw.runOptions.workingdir.input");
            applicationProps.put("javacw.runOptions.workingdir.input", rOptionsArray[5]);
            rOptionsArray[6] = appConfig.getProperty("javacw.runOptions.classname.switch");
            applicationProps.put("javacw.runOptions.classname.switch", rOptionsArray[6]);
            rOptionsArray[7] = appConfig.getProperty("javacw.runOptions.classname.input");
            applicationProps.put("javacw.runOptions.classname.input", rOptionsArray[7]);
            applicationProps.save(out, "---!! Don't mess with .ini-files !!---");
            out.close();
        }
        catch(Exception e)
        {
            System.out.println("Error while writing to file Javacw.ini: "+e);
        }
        finally
        {
            printStatus("saved settings");
        }
    }*/

    /**
    * Saves the compile options to the file Javacw.ini
    
    protected void writeCompileConfig()
    { 
        try
        {
            printStatus("Saving settings to file "+projectName+" ...");
            FileOutputStream out = new FileOutputStream(projectDir+projectName);
            Properties applicationProps = new Properties();
            //save the current compiler settings and update the local backup
            cOptionsArray[0] = appConfig.getProperty("javacw.compilerOptions.g.switch");
            applicationProps.put("javacw.compilerOptions.g.switch", cOptionsArray[0]);
            cOptionsArray[1] = appConfig.getProperty("javacw.compilerOptions.optimize.switch");
            applicationProps.put("javacw.compilerOptions.optimize.switch", cOptionsArray[1]);
            cOptionsArray[2] = appConfig.getProperty("javacw.compilerOptions.nowarn.switch");
            applicationProps.put("javacw.compilerOptions.nowarn.switch", cOptionsArray[2]);
            cOptionsArray[3] = appConfig.getProperty("javacw.compilerOptions.verbose.switch");
            applicationProps.put("javacw.compilerOptions.verbose.switch", cOptionsArray[3]);
            cOptionsArray[4] = appConfig.getProperty("javacw.compilerOptions.deprecation.switch");
            applicationProps.put("javacw.compilerOptions.deprecation.switch",cOptionsArray[4]);
            cOptionsArray[5] = appConfig.getProperty("javacw.compilerOptions.classpath.switch");
            applicationProps.put("javacw.compilerOptions.classpath.switch", cOptionsArray[5]);
            cOptionsArray[6] = appConfig.getProperty("javacw.compilerOptions.classpath.input");
            applicationProps.put("javacw.compilerOptions.classpath.input", cOptionsArray[6]);
            cOptionsArray[7] = appConfig.getProperty("javacw.compilerOptions.autoaddPath.switch");
            applicationProps.put("javacw.compilerOptions.autoaddPath.switch", cOptionsArray[7]);
            cOptionsArray[8] = appConfig.getProperty("javacw.compilerOptions.directory.switch");
            applicationProps.put("javacw.compilerOptions.directory.switch", cOptionsArray[8]);
            cOptionsArray[9] = appConfig.getProperty("javacw.compilerOptions.directory.input");
            applicationProps.put("javacw.compilerOptions.directory.input", cOptionsArray[9]);
            cOptionsArray[10] = appConfig.getProperty("javacw.compilerOptions.depend.switch");
            applicationProps.put("javacw.compilerOptions.depend.switch", cOptionsArray[10]);
            cOptionsArray[11] = appConfig.getProperty("javacw.compilerOptions.j.switch");
            applicationProps.put("javacw.compilerOptions.j.switch", cOptionsArray[11]);
            cOptionsArray[12] = appConfig.getProperty("javacw.compilerOptions.j.input");
            applicationProps.put("javacw.compilerOptions.j.input", cOptionsArray[12]);
            //save the run settings without changes
            applicationProps.put("javacw.runOptions.invokeAllowed.switch", rOptionsArray[0]);
            applicationProps.put("javacw.runOptions.autoaddPath.switch", rOptionsArray[1]);
            applicationProps.put("javacw.runOptions.classpath.switch", rOptionsArray[2]);
            applicationProps.put("javacw.runOptions.classpath.input", rOptionsArray[3]);
            applicationProps.put("javacw.runOptions.workingdir.switch", rOptionsArray[4]);
            applicationProps.put("javacw.runOptions.workingdir.input", rOptionsArray[5]);
            applicationProps.put("javacw.runOptions.classname.switch", rOptionsArray[6]);
            applicationProps.put("javacw.runOptions.classname.input", rOptionsArray[7]);
            applicationProps.save(out, "---!! Don't mess with .ini-files !!---");
            out.close();
        }
        catch(Exception e)
        {
            System.out.println("Error while writing to file Javacw.ini: "+e);
        }
        finally
        {
            printStatus("saved settings");
        }
    }*/

    /**
    * Sets the text in the statusbar<br>
    * @author Yani Ioannou
    */
    private void printStatus(String str)
    {
        //status.setText("Status: "+str);
        status.setText(str);
    }

    /**
    * Writes Strings to cText<br>
    * @author Yani Ioannou
    */
    private void print(String str)
    {
        cText.append(str);
    }

    /**
    * Writes Integers to cText<br>
    * @author Yani Ioannou
    */
    private void print(int i)
    {
        cText.append(""+(char) i);
    }

    /**
    * Writes Bytes to cText<br>
    * @author Yani Ioannou
    */
    private void print(byte i)
    {
        cText.append(""+(char) i);
    }

    /**
    * Calls the java compiler method<br>
    * @param args The arguments for the compiler. The same as in the commandline version<br>
    * @author Yani Ioannou
    */
    private boolean compile(String[] args)
    {
        long start = System.currentTimeMillis();
        try
        {
            sun.tools.javac.Main.main(args);
        }
        catch(FailSecurityException e)
        {
            System.out.println( (System.currentTimeMillis() - start) + "ms");
            return false;
        }
        catch(SuccessSecurityException e)
        {
            System.out.println( (System.currentTimeMillis() - start) + "ms");
            return true;
        }
        catch(Exception e)
        {
            System.out.println("Some error occured: "+e);
            return false;
        }
        return false;
    }

    /**
    * ends the wrapper
    * @author Yani Ioannou
    */
    private void exit(int status)
    {
        if (!projectChanged || askForSave())
        {
            sm.setExit(true);
            System.exit(status);
        }
    }
    
    boolean dontSave;
    private boolean askForSave()
    {
        dontSave = false;
        FLMOkCancelDialog ocd = new FLMOkCancelDialog(javacw, "Continue?", "The project has been changed.\nDo you want to continue\nand lose these changes?", "Yes", "No", null);
        ocd.addOkButtonListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                dontSave = true;
            }
        });
        ocd.show();
        return dontSave;
    }

    /**
    * sets the cOptionsArray
    * @author FReAK La Marsch
    * @param array the new cOptionsArray
    */
    public void setCOptionsArray(String[] array)
    {
        this.cOptionsArray = array;
    }

    /**
    * sets the rOptionsArray
    * @author FReAK La Marsch
    * @param array the new cOptionsArray
    */
    public void setROptionsArray(String[] array)
    {
        this.rOptionsArray = array;
    }

    /**
    * enables the components (mainly the Buttons) given in cFilter
    * and calls updateMenuBarState().
    *
    * @see #updateMenuBarState
    * @author FReAK La Marsch
    * @param cFilter components to be enabled
    */
    private void enableComponents(Component[] cFilter)
    {
        for(int i=0; i<cFilter.length; i++)
        {
            if (cFilter[i] == runButton)
            {
                updateRunButtonState();
            }
            else
            {
                cFilter[i].setEnabled(true);
            }
        }
        updateMenuBarState();
    }

    /**
    * disbles the components (mainly the Buttons) given in cFilter
    * and disables the whole MenuBar
    * @author FReAK La Marsch
    * @param cFilter components to be disabled
    */
    private void disableComponents(Component[] cFilter)
    {
        for(int i=0; i<cFilter.length; i++)
        {
            cFilter[i].setEnabled(false);
        }
        for(int j=0; j<menuBar.getMenuCount(); j++)
        {
            Menu cMenu = menuBar.getMenu(j);
            cMenu.setEnabled(false);
            for(int k=0; k<cMenu.getItemCount(); k++)
            {
                cMenu.getItem(k).setEnabled(false);
            }
        }
    }

    /**
    * Tries to load a <code>class</code>-file and to run it if it is an application.  <br>
    * The path of the compiled files and the classpath of the compiler are used as classpath for the application, depending on the
    * <code>Properties</code> "javacw.runOptions.classpath.switch" and "javacw.runOptions.autoaddPath.switch" stored in appConfig 
    *
    * @param cl name of or path to the .class file that should be loaded and run<br>
    * @see FLMClassRunner
    * @see WaitForThread
    * @author FReAK La Marsch
    */
    private void runClass(String cl)
    {
        //get the classpath
        String classpath = null;
        if (appConfig.getProperty("javacw.runOptions.classpath.switch").equals("on"))
        {//filesClasspath always ends with ";"
            classpath = (appConfig.getProperty("javacw.runOptions.autoaddPath.switch").equals("on") && !filesClasspath.equals("")) ? filesClasspath + appConfig.getProperty("javacw.runOptions.classpath.input") : appConfig.getProperty("javacw.runOptions.classpath.input");
        }

        cr = new FLMClassRunner(this, cl, classpath);       
        Thread waitingThread = new WaitForThread(cr);
        cr.start();
        waitingThread.start();
        running = true;
        printStatus("program started");
        Component[] enableFilter = {runButton};// it's now the kill Button!
        enableComponents(enableFilter);
        Component[] disableFilter = {addButton, compileButton, newButton};
        disableComponents(disableFilter);
        runButton.setLabel("Kill");
    }

    /**
    * SecurityManager for this application.<br>
    * handles the System.exit() calls and grants nearly all policys for the application<br>
    * @author Yani Ioannou, FReAK La Marsch
    */
    class CompilerSecurityManager extends SecurityManager
    {    
        private boolean exit=false;

        public void setExit(boolean exit)
        {
            this.exit=exit;
        }

        public void checkExit(int status) throws SecurityException
        {
            if (exit)
            {
                return;
            }
            if (running)
            {
                /*Java 1.2 :-(
                Frame[] fs = Frame.getFrames();
                for(int i=0; i<fs.length; i++)
                {
                if (i>0)
                {//
                fs[i].dispose();
                }
                }
                throw new ExitSecurityException("System.exit is not allowed to be called by the executed program or the wrapper at this moment");
                */
                /* Java 1.1 */
                return;
            }
            else
            {//this should only be the case when sun.tools.javac.Main finishes
                if (status == 1)
                {
                    throw new FailSecurityException("The compiler exited with errors");
                }
                else
                {
                    throw new SuccessSecurityException("The compiler completed successfully");
                }
            }
        }

        public void checkAccept(String host, int port)
        {
        }
        public void checkAccess(Thread thread)
        {
        }
        public void checkAccess(ThreadGroup threadgroup)
        {
        }
        public void checkConnect(String host, int port)
        {
        }
        public void checkConnect(String host, int port, Object executionContext)
        {
        }
        public void checkCreateClassLoader()
        {
        }
        public void checkDelete(String filename)
        {
        }
        public void checkExec(String command)
        {
        }
        public void checkLink(String library)
        {
        }
        public void checkListen(int port)
        {
        }
        public void checkMemberAccess(Class clazz, int which)
        {
        }
        public void checkPackageAccess(String packageName)
        {
        }
        public void checkPackageDefinition(String packageName)
        {
        }
        public void checkPropertiesAccess()
        {
        }
        public void checkPropertyAccess(String key)
        {
        }
        public void checkPropertyAccess(String key, String def)
        {
        }
        public void checkRead(FileDescriptor filedescriptor)
        {
        }
        public void checkRead(String filename)
        {
        }
        public void checkRead(String filename, Object executionContext)
        {
        }
        public void checkSetFactory()
        {
        }
        public boolean checkTopLevelWindow(Object window)
        {
            return true;
        }
        public void checkWrite(FileDescriptor filedescriptor)
        {
        }
        public void checkWrite(String filename)
        {
        }
        public void createSecurityManager()
        {
        }
        public void checkAwtEventQueueAccess()
        {
        }
        public void checkSecurityAccess(String target)
        {
        }
        public void checkSystemClipboardAccess()
        {
        }
    }


    class SuccessSecurityException extends SecurityException
    {
        SuccessSecurityException()
        {
            super();
        }

        SuccessSecurityException(String message)
        {
            super(message);
        }
    }

    class FailSecurityException extends SecurityException
    {
        FailSecurityException()
        {
            super();
        }

        FailSecurityException(String message)
        {
            super(message);
        }
    }

    class ExitSecurityException extends SecurityException
    {
        ExitSecurityException()
        {
            super();
        }

        ExitSecurityException(String message)
        {
            super(message);
        }
    }

    /**
    * Thread that waits for another thread to die and calls appCleanup() when the thread is dead
    * @author FReAK La Marsch
    */
    class WaitForThread extends Thread
    {
        private Thread threadWaitingFor;

        WaitForThread(Thread t)
        {
            threadWaitingFor = t;
        }

        public void run()
        {
            while (threadWaitingFor.isAlive())
            {
                try
                {
                    threadWaitingFor.join();
                }
                catch(Exception e)
                {
                    //System.out.println(e);
                }
            }
            appCleanup();
        }
    }

    /**
    * Is called to finish an application that has been run by the runButton
    * @author FReAK La Marsch
    */
    public void appCleanup()
    {
        running = false;
        System.gc();
        printStatus("done running program");
        runButton.setLabel("Run");
        Component[] enableFilter = {runButton, addButton, compileButton, newButton};
        enableComponents(enableFilter);
    }

    /**
    * Sets the sourceFiles
    * @author FReAK La Marsch
    * @param v the new sourceFiles <code>FLMResizableArray</code>
    */
    protected void setSourceFiles(FLMResizableArray v)
    {
        sourceFiles = v;
        if (sourceFiles.size() == 0)
        {
            clear();
            menuState.put("RemoveFile", new Boolean(false));
            updateMenuBarState();

        }
    }

    /**
    * Returns sourceFiles
    * @author FReAK La Marsch
    */
    protected FLMResizableArray getSourceFiles()
    {
        return sourceFiles;
    }

    /*
	Could also tak a URL so that getResouce hasn't to be castet from url to string an back again
	private Image getImage(String path)
    {
        try
        {
            java.net.URL url = getClass().getResource(path);
            Image image = Toolkit.getDefaultToolkit().getImage(url);
            return image;
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        return null;
    }*/
}
