import java.awt.*;
import freaklamarsch.awt.*;
import java.awt.event.*;
import java.util.*;
import freaklamarsch.io.*;

/**
* <code>Dialog</code> to set the Run Options for <code>Javacw</code>
* @see Javacw
* @author FReAK La Marsch
*/
public class RunOptions extends Dialog
{
    private Checkbox allowrun, autoadd, classpath, workingdir, classname;
    private TextField classpathInput, workingdirInput, classnameInput;
    private Button setButton, cancelButton, saveButton, classpathInputButton, workingdirInputButton, classnameInputButton;
    private Properties p;
    private Javacw owner;

    private ActionListener setAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            updateSettings();
            owner.setAppConfig(p);
            dispose();
        }
    };

    private ActionListener cancelAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            dispose();
        }
    };

    private ActionListener saveAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            updateSettings();
            owner.setAppConfig(p);
            try
            {
                owner.saveProject("RunOptions");
            }
            catch(Exception e)
            {
                System.out.println("Couldn't save Options because:\n"+e);
            }
            dispose();
        }
    };

    class InputButtonActionListener implements ActionListener
    {
        private TextField inputField;

        InputButtonActionListener(TextField tf)
        {
            inputField = tf;
        }

        public void actionPerformed(ActionEvent evt)
        {
            FLMFileDialog fd = new FLMFileDialog(owner, System.getProperty("user.dir"), "Add path...");
            fd.setFilenameFilter(new FLMFilenameFilter("*.java"), "*.java");
            fd.setVisible(true);
            String inputLabel = inputField.getName();
            String inputValue = inputField.getText();
            String dir = fd.getDirectory();
            if (inputLabel.equals("classpath") || inputLabel.equals("-bootclasspath") || inputLabel.equals("extdirs"))
            {
                String file = fd.getFile();
                if (file.toUpperCase().endsWith("JAR") || file.toUpperCase().endsWith("ZIP"))
                {
                    inputValue = (inputValue.equals("")) ? dir+file : inputValue+";"+dir+file;
                }
                else
                {
                    inputValue = (inputValue.equals("")) ? dir : inputValue+";"+dir;
                }
            }
            else if (inputLabel.equals("workingdir"))
            {
                inputValue = dir;
            }
            else if (inputLabel.equals("classname"))
            {
                String file = fd.getFile();
                inputValue = file.toUpperCase().endsWith(".JAVA") ? dir+file : "";
            }
            inputField.setText(inputValue);
            fd.dispose();
        }
    };

    RunOptions(Javacw ow)
    {
        super(ow, "Run options", true);
        this.owner = ow;
        p = owner.getAppConfig();

        //Listener to close this Dialog without sending data
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                dispose();
            }
        });

        //Add the input elements to this Dialog
        setFont(new Font("Arial", Font.PLAIN, 10));

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(gridbag);

        allowrun = new Checkbox("Allow run by invoke");
        allowrun.setState(p.getProperty("javacw.runOptions.invokeAllowed.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(allowrun, constraints);

        autoadd = new Checkbox("Autobuild classpath");
        autoadd.setState(p.getProperty("javacw.runOptions.autoaddPath.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(autoadd, constraints);

        classpath = new Checkbox("Classpath:");
        classpath.setState(p.getProperty("javacw.runOptions.classpath.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classpath, constraints);

        classpathInput = new TextField(p.getProperty("javacw.runOptions.classpath.input"),10);
        classpathInput.setName("classpath");
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 14;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classpathInput, constraints);

        classpathInputButton = new Button("...");
        classpathInputButton.addActionListener(new InputButtonActionListener(classpathInput));
        constraints.gridx = 14;
        constraints.gridy = 4;
        constraints.gridwidth = 2;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(classpathInputButton, constraints);

        classname = new Checkbox("Class name to run:");
        classname.setState(p.getProperty("javacw.runOptions.classname.switch").equals("on"));
        constraints.gridx = 0;
        constraints.gridy = 7;
        constraints.gridwidth = 16;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classname, constraints);

        classnameInput = new TextField(p.getProperty("javacw.runOptions.classname.input"),10);
        classnameInput.setName("classname");
        constraints.gridx = 0;
        constraints.gridy = 8;
        constraints.gridwidth = 14;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(classnameInput, constraints);

        classnameInputButton = new Button("...");
        classnameInputButton.addActionListener(new InputButtonActionListener(classnameInput));
        constraints.gridx = 14;
        constraints.gridy = 8;
        constraints.gridwidth = 2;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(classnameInputButton, constraints);

        setButton = new Button("set");
        setButton.addActionListener(setAction);
        constraints.gridx = 1;
        constraints.gridy = 9;
        constraints.gridwidth = 4;
        constraints.insets = new Insets(5,2,0,2);
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.fill = GridBagConstraints.NONE;
        add(setButton, constraints);

        cancelButton = new Button("cancel");
        cancelButton.addActionListener(cancelAction);
        constraints.gridx = 5;
        constraints.gridy = 9;
        constraints.gridwidth = 4;
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.fill = GridBagConstraints.NONE;
        add(cancelButton, constraints);

        saveButton = new Button("save");
        saveButton.addActionListener(saveAction);
        constraints.gridx = 9;
        constraints.gridy = 9;
        constraints.gridwidth = 4;
        constraints.anchor = GridBagConstraints.SOUTH;
        constraints.fill = GridBagConstraints.NONE;
        add(saveButton, constraints);

        validate();
        pack();
    }

    private void updateSettings()
    {
        p.put("javacw.runOptions.invokeAllowed.switch", (allowrun.getState()) ? "on" : "off");
        p.put("javacw.runOptions.autoaddPath.switch", (autoadd.getState()) ? "on" : "off");
        p.put("javacw.runOptions.classpath.switch", (classpath.getState()) ? "on" : "off");
        p.put("javacw.runOptions.classpath.input", classpathInput.getText());
        p.put("javacw.runOptions.classname.switch", (classname.getState()) ? "on" : "off");
        p.put("javacw.runOptions.classname.input", classnameInput.getText());
    }
}