package freaklamarsch.awt;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import freaklamarsch.util.*;
import freaklamarsch.io.*;

class FLMDirectoryCanvas extends Canvas implements MouseListener, ItemSelectable
{
    FLMResizableArray files = new FLMResizableArray();
    FLMResizableArray dirs = new FLMResizableArray();
    FLMResizableArray itemListeners = new FLMResizableArray();

    int gapSize = 2;
    int spacing = 0;
    FontMetrics metrics;
    Image image;
    int lineHeight;
    int iconWidth;
    Dimension preferredSize = new Dimension();

    
    //Icons representing Files, Folders and the go to parent dir
    private Image folderIcon;
    private Image fileIcon;
    private Image parentIcon;

    //Stuff to save the current state of the dialog
    private File selectedFile;
    private int selectedIndex;
    private String dir = null;
    private String file = null;
    private FilenameFilter filter;

    FLMDirectoryCanvas(String dir)
    {
        this(new File(dir), new FLMFilenameFilter("*"));
    }

    FLMDirectoryCanvas(String dir, FilenameFilter filter)
    {
        this(new File(dir), filter);
    }

    FLMDirectoryCanvas(File dir)
    {
        this(dir, new FLMFilenameFilter("*"));
    }
    
    FLMDirectoryCanvas(File dir, FilenameFilter filter)
    {
        this.filter = filter;
        loadIcons();
        setFont(new Font("Helvetica", 0, 12));
        metrics = getToolkit().getFontMetrics(getFont());
        lineHeight = Math.max(fileIcon.getHeight(null),Math.max(folderIcon.getHeight(null),metrics.getAscent()+metrics.getDescent()));
        iconWidth = Math.max(fileIcon.getWidth(null),folderIcon.getWidth(null));
        addMouseListener(this);
        setDirectory(dir.getAbsolutePath());
    }	
    

    /*private void calculateSize()
    {
        int width = 0;
        int size = files.size();
        for(int i =0; i<size;i++)
        {
            width = Math.max(width, iconWidth+gapSize+metrics.stringWidth(((File)files.get(i)).getName()));
        }
        size = dirs.size();
        for(int i =0; i<size;i++)
        {
            width = Math.max(width, iconWidth+gapSize+metrics.stringWidth(((File)dirs.get(i)).getName()));
        }
        int height = size*lineHeight;
        preferredSize.setSize(width,height);
        //setSize(preferredSize);
    }*/

    private void loadIcons()
    {
        MediaTracker tracker = new MediaTracker(this);
        folderIcon = Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/Folder_closed.gif"));
        fileIcon = Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/File.gif"));
        parentIcon = Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/Folder_parent.gif"));
        tracker.addImage(folderIcon,0);
        tracker.addImage(fileIcon,1);
        tracker.addImage(parentIcon,2);
        try
        {
            tracker.waitForAll();
        }
        catch(Exception e){System.out.println(e);}
    }

    public void reload()
    {
        setContent(new File(dir));
    }

    /**
    * @param f the File which content should be displayed
    *        if f is a Directory, then the content of this Directory is displayed, otherwise the the content is null
    */
    private void setContent(File f)
    {
        /*clear the current content*/
        dirs = new FLMResizableArray();
        files = new FLMResizableArray();
        file = null;
        selectedIndex = -1;
        if(f==null){return;}
        if (!f.isDirectory())
        {
            dir = null;
            file = f.getAbsolutePath();
            return;
        }
        setDir(f.getAbsolutePath());

        String[] list = f.list();
        if (list == null)
        {
            return;
        }
       
        File temp;
        String currDir = (f.getAbsolutePath().endsWith(File.separator)) ? f.getAbsolutePath() : f.getAbsolutePath()+File.separator;

        for(int i=0; i<list.length; i++)
        {
            temp = new File(currDir+list[i]);

            if (temp.isDirectory())
            {
                dirs.add(temp);
            }
            else if (temp.isFile())
            {//filter the elements
                
                if ((filter == null) || filter.accept(new File(temp.getParent()), temp.getName()))
                {
                	files.add(temp);
                }
            }
        }
        image = null;
        repaint();
        setPreferredSize();
        setSize(preferredSize);
    }

    private void setPreferredSize()
    {
        int width = 0;
        int size = files.size();
        for(int i =0; i<size;i++)
        {
            width = Math.max(width, iconWidth+gapSize+metrics.stringWidth(((File)files.get(i)).getName()));
        }
        size = dirs.size();
        for(int i =0; i<size;i++)
        {
            width = Math.max(width, iconWidth+gapSize+metrics.stringWidth(((File)dirs.get(i)).getName()));
        }
        int height = (files.size()+dirs.size())*lineHeight;
        preferredSize.setSize(width,height);
        //setSize(preferredSize);
    }

    public void createBufferImage()
    {
        preferredSize = new Dimension();
        setPreferredSize();
        image = createImage(preferredSize.width, preferredSize.height);
        Graphics buffer = image.getGraphics();

        int yPos = 0;
        int size = 0;
        Image icon;

        size = dirs.size();
    
        for (int i = 0;i < size;i++)
        {
            icon = folderIcon;
            buffer.drawImage(icon, 0, yPos, this);
            buffer.drawString(((File)dirs.get(i)).getName(), iconWidth+gapSize, yPos+metrics.getAscent());
            yPos+=lineHeight;//evtl auch n Gap
        }

        size = files.size();

        for (int i = 0;i < size;i++)
        {
            icon = fileIcon;
            buffer.drawImage(icon, 0, yPos, this);
            buffer.drawString(((File)files.get(i)).getName(), iconWidth+gapSize, yPos+metrics.getAscent());
            yPos+=lineHeight;//evtl auch n Gap
        }
    }

    public void openParent()
    {
        if(!dir.equals(File.separator) && !dir.equals(""))
        {
            if(dir.endsWith(File.separator))
            {
                setContent(new File(new File(dir.substring(0,dir.length()-2)).getParent()));
            }
            else
            {
                setContent(new File(new File(dir).getParent()));
    	    }
        }
    }

    public Dimension minimumSize()
    {
        return new Dimension(10,10);
    }

    public Dimension preferredSize()
    {
        return getPreferredSize();
    }

    public Dimension getPreferredSize()
    {
        setPreferredSize();
        return preferredSize;
    }

    public void paint(Graphics g)
    {
        if(image == null)
        {
           createBufferImage();
        }
        //g.translate(-xScrollPos,-yScrollPos);
        g.drawImage(image, 0, 0, this);
    }

    private void drawSelection()
    {
        if(selectedFile != null && selectedIndex>=0)
        {
            Graphics g = image.getGraphics();
            g.setColor(Color.blue);
            g.fillRect(0, selectedIndex*lineHeight, getSize().width, lineHeight);
            if(selectedFile.isDirectory())
            {
                g.drawImage(folderIcon, 0, selectedIndex*lineHeight, this);
            }
            else
            {
                g.drawImage(fileIcon, 0, selectedIndex*lineHeight, this);
            }
            g.setColor(Color.white);
            g.drawString(selectedFile.getName(), iconWidth+gapSize, selectedIndex*lineHeight+metrics.getAscent());
            repaint();
        }
    }
        
    private void clearSelection()
    {
        if(selectedFile != null && selectedIndex>=0)
        {
            Graphics g = image.getGraphics();
            g.setColor(Color.white);
            g.fillRect(0, selectedIndex*lineHeight, getSize().width, lineHeight);
            if(selectedFile.isDirectory())
            {
                g.drawImage(folderIcon, 0, selectedIndex*lineHeight, this);
            }
            else
            {
                g.drawImage(fileIcon, 0, selectedIndex*lineHeight, this);
            }
            g.setColor(Color.black);
            g.drawString(selectedFile.getName(), iconWidth+gapSize, selectedIndex*lineHeight+metrics.getAscent());
            repaint();
        }
    }
        
    void selectFile(int yPos)
    {
        clearSelection();
        selectedIndex = yPos/lineHeight;
        selectedFile = (selectedIndex >= dirs.size()) ? ((selectedIndex-dirs.size() >= files.size()) ? null : (File)files.get(selectedIndex-dirs.size())) : (File)dirs.get(selectedIndex);//dirs are always displayed above the files
        if(selectedFile != null)
        {
            setFile(selectedFile);
            if(!selectedFile.isDirectory())
            {
                drawSelection();
            }
        }
    }

    /**
    * Selects a specific file. If the file is not in the current directory, nothing is done
    * If the file is a directory, the file is opened in the canvas
    */
    void selectFile(File f)
    {
        if(files.contains(f))
        {
            for(int i=0;i<files.size();i++)
            {
                if(files.get(i).equals(f))
                {
                    clearSelection();
                    selectedIndex = i;
                    selectedFile = (File)files.get(i);
                    setFile(((File)files.get(i)));
                    drawSelection();
                    break;
                }                
            }
        }
        else if(dirs.contains(f))
        {
            for(int i=0;i<dirs.size();i++)
            {
                if(dirs.get(i).equals(f))
                {
                    clearSelection();
                    selectedIndex = i;
                    selectedFile = (File)dirs.get(i);
                    setFile(((File)dirs.get(i)));
                    setContent((File)dirs.get(i));
                    drawSelection();
                    break;
                }                
            }
        }
        else if(f.isDirectory())
        {
            clearSelection();
            selectedIndex = -1;
            selectedFile = f;
            setFile(new File(""));
            setContent(f);
        }
    }

    /**
     * Gets the directory of this file dialog.
     * @return    the directory of this file dialog.
     */
    public String getDirectory() {
        return dir;
    }

    /**
     * Sets the directory of this file dialog window to be the  
     * specified directory. 
     * @param     dir   the specific directory.
     */
    public void setDirectory(String dir)
    {
        setDir(dir);
        if(dir != null)
        {//you need this if-else because you can't creat a File with null als argument
            setContent(new File(dir));
        }
        else setContent(null);
    }

    private void setDir(String dir)
    {
        try
        {
            this.dir = dir.endsWith(File.separator) ? dir : dir+File.separator;
        }
        catch(NullPointerException e)
        {
            this.dir = null;
        }
    }

    /**
     * Gets the selected file of this file dialog.
     * @return    the currently selected file of this file dialog window, 
     *                or <code>null</code> if none is selected.
     */
    public String getFile() {
        return file;
    }

    /**
     * Sets the selected file for this file dialog window to be the 
     * specified file. This file becomes the default file if it is set 
     * before the file dialog window is first shown. 
     * @param    file   the file being set.
     
    public void setFile(String file)
    {
        try
        {
            this.file = file;
            stateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, new File(file), ItemEvent.SELECTED));
        }
        catch(NullPointerException npe)
        {
            this.file = null;
            stateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, null, ItemEvent.DESELECTED));
        }
        System.out.println("FILE = "+this.file);
    }*/
    
    public void setFile(File file)
    {
        try
        {
            this.file = file.getName();
            stateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, file, ItemEvent.SELECTED));
        }
        catch(NullPointerException npe)
        {
            this.file = null;
            stateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, null, ItemEvent.DESELECTED));
        }
        //System.out.println("FILE = "+this.file);
    }
    
    /**
     * Determines this file dialog's filename filter. A filename filter 
     * allows the user to specify which files appear in the file dialog 
     * window. 
     * @return    this file dialog's filename filter.
     */
    public FilenameFilter getFilenameFilter()
    {
        return filter;
    }

    /**
    * Sets the filename filter for this file dialog window to the 
    * specified filter. 
    * @param   filter   the specified filter.
    */
    public synchronized void setFilenameFilter(FilenameFilter filter)
    {
        this.filter = filter;
        try
        {
            setContent(new File(dir));
        }
        catch(NullPointerException npe){/*this should never happen*/System.out.println("FLMFileDialog: "+npe);}
    }

    public Image getFolderIcon()
    {
        return folderIcon;
    }

    public Image getFileIcon()
    {
        return fileIcon;
    }

    public Image getParentIcon()
    {
        return parentIcon;
    }

    /*public void setSelectedFile(File aFile)
    {
        selectedFile = aFile;
    }*/


   /* MouseListener Stuff*/
    public void mouseClicked(MouseEvent e)
    {
    }
    public void mouseEntered(MouseEvent e)
    {
    }
    public void mouseExited(MouseEvent e)
    {
    }
    public void mousePressed(MouseEvent e)
    {
        selectFile(e.getY());
    }
    public void mouseReleased(MouseEvent e)
    {
    }

    private static MenuBar menuBar;
    private static ActionListener exitAction = new ActionListener()
    {
        public void actionPerformed(ActionEvent evt)
        {
            System.exit(0);
        }
    };

/*----------------ItemSelectable Methods------------------------*/
    public void addItemListener(ItemListener il)
    {
        if(!itemListeners.contains(il))
        {
            itemListeners.add(il);
        }
    }

    public void removeItemListener(ItemListener il)
    {
        if(itemListeners.contains(il))
        {
            for(int i=0;i<itemListeners.size();i++)
            {
                if(itemListeners.get(i).equals(il)) break;
            }
        }
    }
    
    public Object[] getSelectedObjects()
    {
        return new Object[]{selectedFile};
    }

/*------------------------------------------------------------*/

    private void stateChanged(ItemEvent e)
    {
        for(int i=0;i<itemListeners.size();i++)
        {
            ((ItemListener)itemListeners.get(i)).itemStateChanged(e);
        }
    }

/* main is just for testing */
    public static void main(String[] args)
    {
        Frame f = new Frame();
        FLMDirectoryCanvas dc = new FLMDirectoryCanvas(System.getProperty("user.dir"));
        f.add(dc);
        f.setSize(200,300);
        menuBar = new MenuBar();
        Menu fileMenu = new Menu("File");
        fileMenu.setName("File");
        MenuItem exitMenuItem = new MenuItem("Exit");
        exitMenuItem.setName("Exit");
        exitMenuItem.addActionListener(exitAction);
        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        f.setMenuBar(menuBar);
        f.show();
    }
}
