package freaklamarsch.awt;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import freaklamarsch.io.*;


/**
* Alternative implementation of <code>FileDialog</code> that enables <code>FilenameFilter</code>s for
* all platforms.
* At the moment it's a work in progress and doesn't implement all methods from <code>java.awt.FileDialog</code>
* @author FReAK La Marsch
*/
public class FLMFileDialog extends Dialog// implements MouseListener
{
    //Icon representing the go to parent dir
    private Image parentIcon;

    //Stuff to save the current state of the dialog
    public static final int LOAD = 0;
    public static final int SAVE = 1;
    private int mode = LOAD;
    private static int nameCounter = 0;
    boolean overwrite = false;
    Frame parent;

	//Stuff to display the FileDialog
    private TextField currentSelection;
    private Choice filterChoice;
    private FLMScrollPane contentScroller;

	/*Stuff for the Panel which displays the files*/
	private FLMDirectoryCanvas canvas = null;
	    
    public FLMFileDialog(Frame parent)
    {
        this(parent, System.getProperty("user.dir"), "", null, LOAD);
    }

    public FLMFileDialog(Frame parent, String[] filterString)
    {
        this(parent, System.getProperty("user.dir"), "", filterString, LOAD);
    }

    public FLMFileDialog(Frame parent, int mode)
    {
        this(parent, System.getProperty("user.dir"), "", null, mode);
    }

    public FLMFileDialog(Frame parent, String[] filterString, int mode)
    {
        this(parent, System.getProperty("user.dir"), "", filterString, mode);
    }

    public FLMFileDialog(Frame parent, String dir)
    {
        this(parent, dir, "", null, LOAD);
    }

    public FLMFileDialog(Frame parent, String dir, String[] filterString)
    {
        this(parent, dir, "", filterString, LOAD);
    }

    public FLMFileDialog(Frame parent, String dir, int mode)
    {
        this(parent, dir, "", null, mode);
    }

    public FLMFileDialog(Frame parent, String dir, String[] filterString, int mode)
    {
        this(parent, dir, "", filterString, mode);
    }

    public FLMFileDialog(Frame parent, String dir, String title)
    {
        this(parent, dir, title, null, LOAD);
    }

    public FLMFileDialog(Frame parent, String dir, String title, String[] filterStrings)
    {
        this(parent, dir, title, filterStrings, LOAD);
    }

    public FLMFileDialog(Frame parent, String dir, String title, String[] filterStrings, int mode)
    {
        super(parent, title, true);
        this.parent = parent;
        this.mode = mode;
        initComponents(dir, filterStrings);
    }

    private void initComponents(String dir, String[] filterStrings)
    {
        MediaTracker tracker = new MediaTracker(this);
        parentIcon = Toolkit.getDefaultToolkit().getImage(getClass().getResource("images/Folder_parent.gif"));
        tracker.addImage(parentIcon,0);
        try
        {
            tracker.waitForAll();
        }
        catch(Exception e){}

        /* set the size for this dialog */
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setSize(screenSize.width, screenSize.height);

        /* add windowlistener */
        WindowListener windowListener = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                doCancel();
            }
        };
        addWindowListener(windowListener);

        /* create the components of this dialog */
        
        /* tool bar */
        Panel controlPanel = new Panel();
        controlPanel.setLayout(new GridBagLayout());

        GridBagConstraints constraints = new GridBagConstraints();

        /* TextField to display the current folders path */
        currentSelection = new TextField(dir);
        //currentSelection.setEditable(false);
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 7;
        constraints.gridheight = 1;
        constraints.weightx = 1;
        constraints.weighty = 0;
        constraints.insets = new Insets(5, 0, 5, 2);
        constraints.fill = GridBagConstraints.BOTH;
        controlPanel.add(currentSelection, constraints);

        /* Icon to change to the parent folder */
        FLMImageCanvas parentIconCanvas = new FLMImageCanvas(parentIcon);
        parentIconCanvas.addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                try
                {	
                    canvas.openParent();
                    currentSelection.setText(canvas.getDirectory());
                    contentScroller.validate();
                }
                catch(NullPointerException npe){}
            }
        });
        constraints.gridx = 7;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.insets = new Insets(5, 3, 5, 2);
        constraints.fill = GridBagConstraints.NONE;
        controlPanel.add(parentIconCanvas, constraints);
        
        /* Choice to choose the FileNameFilter */
        filterChoice = new Choice();
        FLMFilenameFilter filter;
        if (filterStrings!=null)
       {
            filter = new FLMFilenameFilter(filterStrings[0]);

            for(int i=0;i<filterStrings.length;i++)
            {
                filterChoice.add(filterStrings[i]);
            }
            
        }
        else
        {
            filter = new FLMFilenameFilter("*");
            filterChoice.add("*");
        }
        filterChoice.select(0);

        filterChoice.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                canvas.setFilenameFilter(new FLMFilenameFilter(filterChoice.getSelectedItem()));
                canvas.reload();
                contentScroller.validate();
            }
        });

        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.weightx = 1;
        constraints.weighty = 0;
        constraints.insets = new Insets(0, 3, 5, 2);
        constraints.fill = GridBagConstraints.HORIZONTAL;
        controlPanel.add(filterChoice, constraints);

        /* Button to hide this FileDialog */
        Button okButton = (mode == LOAD) ? new Button("Open") : new Button("Save");
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                try
                {
                    String cs = currentSelection.getText();
                    if(cs.lastIndexOf(File.separator) != -1)
                    {
                        if((new File(cs)).isDirectory())
                        {
                            setDirectory(cs);
                            setFile(null);
                        }
                        else
                        {               
                            setDirectory(cs.substring(0,cs.lastIndexOf(File.separator)));
                            setFile(cs.substring(cs.lastIndexOf(File.separator)+1));
                        }
                    }
                    else
                    {
                       setDirectory("") ;
                       setFile(cs);
                    }
                }
                catch(NullPointerException npe){}
                checkOverwrite();
            }
        });
        constraints.gridx = 2;
        constraints.gridy = 1;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.insets = new Insets(0, 3, 5, 2);
        constraints.fill = GridBagConstraints.NONE;
        controlPanel.add(okButton, constraints);

        /* button to cancel this FileDialog*/
        Button cancelButton = new Button("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e){
                doCancel();
            }
        });
        constraints.gridx = 5;
        constraints.gridy = 1;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.insets = new Insets(0, 3, 5, 0);
        constraints.fill = GridBagConstraints.NONE;
        controlPanel.add(cancelButton, constraints);

        /*Canvas for displaying the content of the current directory*/
        canvas = new FLMDirectoryCanvas(dir, new FLMFilenameFilter(filterChoice.getSelectedItem()));
        canvas.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                try
                {
                    File changedItem = (File)e.getItem();
                    //System.out.println("Something selected! "+changedItem);
                    //System.out.println(changedItem.getAbsolutePath());
                    if(changedItem.isDirectory())
                    {//change into selected file
                        canvas.setDirectory(changedItem.getAbsolutePath());
                        currentSelection.setText(changedItem.getAbsolutePath());
                    }
                    else
                    {//open selected file
                        doOk();
                    }
                }
                catch(NullPointerException npe)
                {
                    //this means that file has been set to null in canvas
                }
            }
        });
        /* ScrollPanel which displays the folder content */
        contentScroller = new FLMScrollPane(FLMScrollPane.CENTER_NONE);//den richtigen muss ich noch einbauen
        contentScroller.addComponent(canvas);
        contentScroller.setBackground(Color.white);
        /*if this line is active the content will be added twice to the scroller
		because Javacw adds a new filenamefilter and this causes the content to reload
		see setFilenameFilter()
        contentScroller.addComponent(content);*/
        
        /* add Tool Bar and ScrollPanel to this dialog */
        setLayout(new GridBagLayout());        

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.weightx = 1;
        constraints.weighty = 0;
        constraints.insets = new Insets(0, 0, 0, 0);
        constraints.fill = GridBagConstraints.HORIZONTAL;
        add(controlPanel, constraints);
        
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.weightx = 1;
        constraints.weighty = 1;
        constraints.insets = new Insets(0, 0, 0, 0);
        constraints.fill = GridBagConstraints.BOTH;
        add(contentScroller, constraints);
    }

    private void doCancel()
    {
        setFile(null);
        setDirectory(null);
        dispose();
    }
    
    private void doOk()
    {
        dispose();
    }

    public int getMode()
    {
        return mode;
    } 

    public void setMode(int mode)
    {
        if (mode == LOAD || mode == SAVE)
        {
            this.mode = mode;
        }
        else
        {
            throw new IllegalArgumentException("illegal file dialog mode");
        }
    }
   
    /**
     * Gets the directory of this file dialog.
     * @return    the directory of this file dialog.
     */
    public String getDirectory() {
        return canvas.getDirectory();
    }

    /**
     * Sets the directory of this file dialog window to be the  
     * specified directory. 
     * @param     dir   the specific directory.
     */
    public void setDirectory(String dir)
    {
        canvas.setDirectory(dir);
    }

    /**
     * Gets the selected file of this file dialog.
     * @return    the currently selected file of this file dialog window, 
     *                or <code>null</code> if none is selected.
     */
    public String getFile() {
        return canvas.getFile();
    }

    /**
     * Sets the selected file for this file dialog window to be the 
     * specified file. This file becomes the default file if it is set 
     * before the file dialog window is first shown. 
     * @param    file   the file being set.
     */
    public void setFile(String file) {
        if(file!=null)
        {
            canvas.setFile(new File(file));
        }
        else canvas.setFile(null);
    }
    
    /**
     * Determines this file dialog's filename filter. A filename filter 
     * allows the user to specify which files appear in the file dialog 
     * window. 
     * @return    this file dialog's filename filter.
     */
    public FilenameFilter getFilenameFilter()
    {
        return canvas.getFilenameFilter();
    }

    /**
    * Sets the filename filter for this file dialog window to the 
    * specified filter. 
    * @param   filter   the specified filter.
    */
    public synchronized void setFilenameFilter(FilenameFilter filter, String filterString)
    {
        canvas.setFilenameFilter(filter);
        if (filterChoice.getItemCount()>1)
        {
            filterChoice.remove(0);
        }
        filterChoice.insert(filterString,0);
        filterChoice.select(0);
    }

    private void checkOverwrite()
    {
        if (mode == SAVE && new File(getDirectory()+getFile()).exists())
        {
            FLMOkCancelDialog ocd = new FLMOkCancelDialog(parent, "Overwrite?", "The file \n"+getFile()+"\nalready exists!\nDo you want to overwrite that file?", "Yes", "No", null);
            ocd.addOkButtonListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    overwrite = true;
                }
            });
            ocd.show();
            if (overwrite)
            {
                overwrite = false;
                dispose();
            }
        }
        else
        {
            dispose();
        }
    }

    public void selectFile(File aFile)
    {
        canvas.selectFile(aFile);
        currentSelection.setText(canvas.getDirectory());
    }
}


