package freaklamarsch.awt;

import java.awt.*;
import java.net.*;

/**
* Provides an image inside of a canvas
* The canvas is sized to fit the image exactly.
* @author FReAK La Marsch
*/
public class FLMImageCanvas extends Canvas
{

    private Image image;

    /**
     * Constructor
     * @param image the filename of the <code>Image</code> which should be displayed by this canvas
     */
    public FLMImageCanvas(String image)
    {
        this(Toolkit.getDefaultToolkit().getImage(image));
    }

    /**
     * Constructor
     * @param image the URL of the <code>Image</code> which should be displayed by this canvas
     */
    public FLMImageCanvas(URL image)
    {
        this(Toolkit.getDefaultToolkit().getImage(image));
    }

    /**
     * Constructor
     * @param image the <code>Image</code> which should be displayed by this canvas
     */
    public FLMImageCanvas(Image image)
    {
        this.image = image;
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);

        try
        {
            tracker.waitForID(0);
            setSize(image.getWidth(this), image.getHeight(this));
        }
        catch(InterruptedException e)
        {
            System.out.println("Can`t load the image");
        }
    }

    public void paint(Graphics g)
    {
        g.drawImage(image,0, 0, this);
    }
}