package freaklamarsch.awt;

import java.awt.*;
import java.net.*;
import java.awt.event.*;

/**
* Simple <code>Dialog</code> to ask the user a Question with two possible answers
* @author FReAK La Marsch
*/
public class FLMOkCancelDialog extends Dialog
{
    private String message;
    private String okButtonLabel;
    private String cancelButtonLabel;
    private Panel messagePanel;

    private WindowListener windowListener = new WindowAdapter() { public void windowClosing(WindowEvent e) {dispose();} };

    private ActionListener disposeAction = new ActionListener() { public void actionPerformed(ActionEvent event) {dispose();} };

    private Button okButton;
    private Button cancelButton;
    private FLMDialogIcon icon;

    public FLMOkCancelDialog(Frame owner, String m)
    {
        this(owner, "", m, "OK", "Cancel", null);
    }

    public FLMOkCancelDialog(Frame owner, String m, URL icon)
    {
        this(owner, "", m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String m, Image icon)
    {
        this(owner, "", m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String title, String m)
    {
        this(owner, title, m, "OK", "Cancel", null);
    }

    public FLMOkCancelDialog(Frame owner, String title, String m, URL icon)
    {
        this(owner, title, m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String title, String m, Image icon)
    {
        this(owner, title, m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String m, String obl, String cbl)
    {
        this(owner, "", m, "OK", "Cancel", null);
    }

    public FLMOkCancelDialog(Frame owner, String m, String obl, String cbl, URL icon)
    {
        this(owner, "", m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String m, String obl, String cbl, Image icon)
    {
        this(owner, "", m, "OK", "Cancel", new FLMDialogIcon(icon));
    }

    public FLMOkCancelDialog(Frame owner, String title, String m, String obl, String cbl)
    {
        this(owner, title, m, "OK", "Cancel", null);
    }

    public FLMOkCancelDialog(Frame owner, String title, String m, String obl, String cbl, FLMDialogIcon di)
    {
        super(owner, title, true);
        message = m;
        okButtonLabel = obl;
        cancelButtonLabel = cbl;
        icon = di;
        initComponents();
    }

    private void initComponents()
    {
        setLayout(new BorderLayout(5,5));
        addWindowListener(windowListener);
  
        Panel spacer = new Panel();
        if (icon != null)
        {
            add(icon, BorderLayout.WEST);
        }
        
        messagePanel = createMessagePanel(message);
        add(messagePanel, BorderLayout.CENTER);        
  
        // Buttons
        Panel buttonPanel = new Panel();
        buttonPanel.setLayout (new FlowLayout());
        okButton = new Button();
        okButton.setLabel(okButtonLabel);
        okButton.addActionListener(disposeAction);
        buttonPanel.add(okButton);
    
        // spacing
        buttonPanel.add(spacer);

        cancelButton = new Button();
        cancelButton.setLabel(cancelButtonLabel);
        cancelButton.addActionListener(disposeAction);
        buttonPanel.add(cancelButton);
  
        add(buttonPanel, BorderLayout.SOUTH);

        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        move((screenSize.width-getSize().width)/2,(screenSize.height-getSize().height)/2);
    }

    private Panel createMessagePanel(String message)
    {
        Panel panel = new Panel();
        GridBagLayout gbLayout = new GridBagLayout();
        panel.setLayout(gbLayout);

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.weightx  = 1.0;
        constraints.weighty  = 1.0;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.insets   = new Insets(0,0,0,0);

        while (message.length() > 0)
        {
            int lineEnd = message.indexOf('\n');
            String line;
            if (lineEnd >= 0)
            {
                line = message.substring(0, lineEnd);
                message = message.substring(lineEnd + 1);
            }
            else
            {
                line = message;
                message = "";
            }
            Label label = new Label(line);
            panel.add(label, constraints);
        }
        return panel;
    }
    
    public void addCancelButtonListener(ActionListener cbl)
    {
        cancelButton.addActionListener(cbl);
    }

    public void addOkButtonListener(ActionListener obl)
    {//this ensures (at least I hope it does), that the disposeAction always handles the Event at last.
        okButton.removeActionListener(disposeAction);
        okButton.addActionListener(obl);
        okButton.addActionListener(disposeAction);
    }

    /**
    * Method to access the inner class FLMDialogIcon
    */
    public static FLMDialogIcon newFLMDialogIcon(URL icon)
    {
        return new FLMDialogIcon(icon);
    }

    /**
    * Method to access the inner class FLMDialogIcon
    */
    public static FLMDialogIcon newFLMDialogIcon(String icon)
    {
        return new FLMDialogIcon(icon);
    }
    /**
    * Method to access the inner class FLMDialogIcon
    */
    public static FLMDialogIcon newFLMDialogIcon(Image icon)
    {
        return new FLMDialogIcon(icon);
    }
}


/**
* Panel that has an <code>FLMImageCanvas</code> in the middle and some empty <code>Panel</codes>s around the image
* When the Panel is resized, the image will keep it's size and stay centered in the Panel.
* @see FLMImageCanvas
* @author FReAK La Marsch
*/
class FLMDialogIcon extends Panel
{
    FLMImageCanvas ic;
    
    FLMDialogIcon(Image icon)
    {
    }

    FLMDialogIcon(URL icon)
    {
        ic = new FLMImageCanvas(Toolkit.getDefaultToolkit().getImage(icon));
    }
    
    FLMDialogIcon(String icon)
    {
        ic = new FLMImageCanvas(Toolkit.getDefaultToolkit().getImage(icon));
    }
    
    private void initComponents()
    {    
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);

        Panel spacer = new Panel();

        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints(spacer, c);
        add(spacer);
       
        c.gridx = 0;
        c.gridy = 1;
        c.fill = GridBagConstraints.NONE;
        gridbag.setConstraints(ic, c);
        add(ic);
       
        c.gridx = 0;
        c.gridy = 2;
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints(spacer, c);
        add(spacer);
    }
}