package freaklamarsch.awt;

import java.awt.*;
 
/**
* <code>FLMScrollPane</code> subclasses <code>ScrollPane</code> and adds some
* bugfixes, workaraounds and new behaviour.
* @author FReAK La Marsch
*/

public class FLMScrollPane extends ScrollPane
{
	private Panel contentPanel;

    private GridBagConstraints gbc;

    public static final int CENTER_NONE = 0;

    public static final int CENTER_BOTH = 1;

    public static final int CENTER_HORIZONTAL = 2;

    public static final int CENTER_VERTICAL = 3;

    public static final int NORTH_NONE = 10;

    public static final int NORTH_BOTH = 11;

    public static final int NORTH_HORIZONTAL = 12;

    public static final int NORTH_VERTICAL = 13;

    public static final int NORTHEAST_NONE = 20;

    public static final int NORTHEAST_BOTH = 21;

    public static final int NORTHEAST_HORIZONTAL = 22;

    public static final int NORTHEAST_VERTICAL = 23;

    public static final int EAST_NONE = 30;

    public static final int EAST_BOTH = 31;

    public static final int EAST_HORIZONTAL = 32;

    public static final int EAST_VERTICAL = 33;

    public static final int SOUTHEAST_NONE = 40;

    public static final int SOUTHEAST_BOTH = 41;

    public static final int SOUTHEAST_HORIZONTAL = 42;

    public static final int SOUTHEAST_VERTICAL = 43;

    public static final int SOUTH_NONE = 50;

    public static final int SOUTH_BOTH = 51;

    public static final int SOUTH_HORIZONTAL = 52;

    public static final int SOUTH_VERTICAL = 53;

    public static final int SOUTHWEST_NONE = 60;

    public static final int SOUTHWEST_BOTH = 61;

    public static final int SOUTHWEST_HORIZONTAL = 62;

    public static final int SOUTHWEST_VERTICAL = 63;

    public static final int WEST_NONE = 70;

    public static final int WEST_BOTH = 71;

    public static final int WEST_HORIZONTAL = 72;

    public static final int WEST_VERTICAL = 73;

    public static final int NORTHWEST_NONE = 80;

    public static final int NORTHWEST_BOTH = 81;

    public static final int NORTHWEST_HORIZONTAL = 82;

    public static final int NORTHWEST_VERTICAL = 83;

    /**
    * Create a new scrollpane container with a scrollbar display policy of
    * "as needed".
    */
    public FLMScrollPane()
    {
        this(SCROLLBARS_AS_NEEDED, CENTER_BOTH);
    }

    /**
    * Create a new scrollpane container with a given scrollbar display policy
    * @param scrollbarDisplayPolicy policy for when scrollbars should be shown
    */
    public FLMScrollPane(int scrollbarDisplayPolicy)
    {
        this(scrollbarDisplayPolicy, CENTER_BOTH);
    }

    /**
    * Create a new scrollpane container.
    * @param fill policy for how the content should be resized in the contentPanel
    * @param scrollbarDisplayPolicy policy for when scrollbars should be shown
    */
    public FLMScrollPane(int scrollbarDisplayPolicy, int fill)
    {
        super(scrollbarDisplayPolicy);
        //contentPanel = new FLMBufferedPanel(){
        contentPanel = new Panel(){
            public Dimension getPreferredSize()
            {//workaraound for a bug, when the HScrollBar hides the last row of the displayed content, so we size the content a little greater
                Dimension panelPreferredSize = super.getPreferredSize(); 
                //System.out.println("Size = "+getSize());
                //System.out.println("panelPreferredSize = "+ super.getPreferredSize());
                //System.out.println("Preferred Size = "+ ((panelPreferredSize.height > (getSize().height - getInsets().top * 2)) ? new Dimension(panelPreferredSize.width, panelPreferredSize.height + getHScrollbarHeight()) : super.getPreferredSize()));
                return (panelPreferredSize.height > (getSize().height - getInsets().top * 2)) ? new Dimension(panelPreferredSize.width, panelPreferredSize.height + getHScrollbarHeight()) : super.getPreferredSize(); 
            }
        };
        //contentPanel.addComponentListener(this);
        int lineHeight = 16;//this should be calculated
        getVAdjustable().setUnitIncrement(lineHeight);
        getHAdjustable().setUnitIncrement(lineHeight);
        GridBagLayout gridbag = new GridBagLayout();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        setFill(fill);
        contentPanel.setLayout(gridbag);
    }

    private void setFill(int fill)
    {
        if (fill == CENTER_NONE)
        {
            gbc.anchor = GridBagConstraints.CENTER;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == CENTER_BOTH)
        {
            gbc.anchor = GridBagConstraints.CENTER;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == CENTER_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.CENTER;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == CENTER_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.CENTER;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == NORTH_NONE)
        {
            gbc.anchor = GridBagConstraints.NORTH;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == NORTH_BOTH)
        {
            gbc.anchor = GridBagConstraints.NORTH;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == NORTH_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.NORTH;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == NORTH_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.NORTH;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == NORTHEAST_NONE)
        {
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == NORTHEAST_BOTH)
        {
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == NORTHEAST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == NORTHEAST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.NORTHEAST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == EAST_NONE)
        {
            gbc.anchor = GridBagConstraints.EAST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == EAST_BOTH)
        {
            gbc.anchor = GridBagConstraints.EAST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == EAST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.EAST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == EAST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.EAST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == SOUTHEAST_NONE)
        {
            gbc.anchor = GridBagConstraints.SOUTHEAST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == SOUTHEAST_BOTH)
        {
            gbc.anchor = GridBagConstraints.SOUTHEAST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == SOUTHEAST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.SOUTHEAST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == SOUTHEAST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.SOUTHEAST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == SOUTH_NONE)
        {
            gbc.anchor = GridBagConstraints.SOUTH;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == SOUTH_BOTH)
        {
            gbc.anchor = GridBagConstraints.SOUTH;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == SOUTH_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.SOUTH;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == SOUTH_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.SOUTH;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == SOUTHWEST_NONE)
        {
            gbc.anchor = GridBagConstraints.SOUTHWEST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == SOUTHWEST_BOTH)
        {
            gbc.anchor = GridBagConstraints.SOUTHWEST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == SOUTHWEST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.SOUTHWEST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == SOUTHWEST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.SOUTHWEST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == WEST_NONE)
        {
            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == WEST_BOTH)
        {
            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == WEST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == WEST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
        else if (fill == NORTHWEST_NONE)
        {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.fill = GridBagConstraints.NONE;
        }
        else if (fill == NORTHWEST_BOTH)
        {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.fill = GridBagConstraints.BOTH;
        }
        else if (fill == NORTHWEST_HORIZONTAL)
        {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.fill = GridBagConstraints.HORIZONTAL;
        }
        else if (fill == NORTHWEST_VERTICAL)
        {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.fill = GridBagConstraints.VERTICAL;
        }
    }

    /** 
    * Adds the specified component to the end of this <code>FLMScrollPane</code>'s contentPanel. 
    * @param     comp   the component to be added.
    * @return    the component argument.     
    */
    public Component addComponent(Component comp)
    {
        gbc.gridy++;
        contentPanel.add(comp, gbc);
        addImpl(contentPanel, null, -1);
        return comp;
    }

    /**
    * Adds the specified component to the end of this <code>FLMScrollPane</code>'s contentPanel. 
    * It is strongly advised to use the 1.1 method, add(Component, Object),
    * in place of this method.
    */
    public Component addComponent(String name, Component comp)
    {
        gbc.gridy++;
        contentPanel.add(comp, gbc);
        addImpl(contentPanel, name, -1);
        return comp;
    }

    /** 
    * Adds the specified component to the end of this <code>FLMScrollPane</code>'s
    * contentPanel at the given position. 
    * @param     comp   the component to be added.
    * @param     index    the position at which to insert the component,
    *            or <code>-1</code> to insert the component at the end.
    * @return    the component <code>comp</code>
    */
    public Component addComponent(Component comp, int index)
    {
        gbc.gridy = index;
        contentPanel.add(comp, gbc);
        addImpl(contentPanel, null, -1);
        return comp;
    }

    /**
    * Adds the specified component to the end of this <code>FLMScrollPane</code>'s contentPanel. 
    * Also notifies the contentPanel's LayoutManager to add the component to 
    * the contentPanel's layout using the specified constraints object.
    * @param     comp the component to be added
    * @param     constraints an object expressing layout contraints for this component
    */
    public void addComponent(Component comp, Object constraints)
    {
        contentPanel.add(comp, constraints);
        addImpl(contentPanel, null, -1);
    }

    /**
    * Adds the specified component to this <code>FLMScrollPane</code>'s contentPanel 
    * with the specified constraints at the specified index.
    * Also notifies the layout manager to add the component to the this contentPanel's
    * layout using the specified constraints object.
    * @param comp the component to be added
    * @param constraints an object expressing layout contraints for this
    * @param index the position in the container's list at which to insert
    * the component. -1 means insert at the end.
    */
    public void addComponent(Component comp, Object constraints, int index)
    {
        contentPanel.add(comp, constraints, index);
        addImpl(contentPanel, null, -1);
    }

    /** 
    * Adds the specified component to this <code>FLMScrollPane</code>'s contentPanel.
    * If the contentPanel has an existing child component, that
    * component is removed and the new one is added.  
    * @param comp the component to be added 
    * @param constraints  not applicable
    * @param index position of child component (must be <= 0) 
    */
    public void replaceContent(Component comp)
    {
        if (comp!=null)
        {
            contentPanel.removeAll();
            gbc.gridy = 0;
            contentPanel.add(comp, gbc);
            addImpl(contentPanel, null, -1);
            doLayout();
            contentPanel.validate();
            contentPanel.repaint();
        }
    }

    /** 
    * Removes the component, specified by <code>index</code>, 
    * from this <code>FLMScrollPane</code>'s contentPanel. 
    * @param     index   the index of the component to be removed.
    * @see #add
    */
    public void removeComponentAt(int index)
    {
        contentPanel.remove(index);
    }

    /** 
    * Removes the specified component from this <code>FLMScrollPane</code>'s contentPanel.
    * @param comp contentPanel to be removed
    */
    public void removeComponent(Component comp)
    {
        contentPanel.remove(comp);
    }

    /** 
    * Removes all the components from this <code>FLMScrollPane</code>'s contentPanel.
    */
    public void removeAllComponents()
    {
        contentPanel.removeAll();
    }

    public Panel getContentPanel()
    {
        return contentPanel;
    }
}
