package freaklamarsch.io;

import java.io.*;
import freaklamarsch.util.*;
/**
* <code>FilenameFilter</code> that deals with * as wildcards
* @author FReAK La Marsch
*/
public class FLMFilenameFilter implements FilenameFilter
{
    private String filterString;

    /**
    * Constructor
    * @param extension String for which the file should be filtered
    * 
    * Allowed syntax for the filter are *, bla*, *bla*, *bla, *.*, *.ext, bla.*,
    * bla*.*, *bla*.*, *bla.*, bla*.ext, *bla*.ext, *bla.ext, bla, bla.ext, etc. :-)
    */
    public FLMFilenameFilter(String filterString)
    {
    	this.filterString = filterString;
    }

    public String getFilterString()
    {
        return filterString;
    }
    
    public void setFilterString(String filterString)
    {
        this.filterString = filterString;
    }
    
    /**
    * Tests if a specified file should be included in a file list.
    * @param dir the directory in which the file was found.
    * @param name the name of the file.
    * @return true if and only if the name should be included in the file list; false otherwise.
    */
    public boolean accept(File dir, String name)
    {
    	if (filterString.equals("*"))
        {
            return true;
        }
        else if (filterString.equals("*.*"))
        {
            return (name.indexOf(".")!=-1);
        }
        else if (filterString.indexOf("*") == -1)
        {
            return name.equalsIgnoreCase(filterString);
        }
        else
        {
            //now we have to search the extension for occurences of '*'
            FLMResizableArray stars = new FLMResizableArray(); //stars will keep the indexes of stars
            char[] filterChars = filterString.toCharArray();
            for(int i=0; i<filterChars.length;i++)
            {
                if (filterChars[i] == '*')
                {
                    stars.add(new Integer(i));
                }
            }
            //weve handled the case that there`s no * already
           /* if there's one *, we have a filter like
            bla*, *bla, *bla.ext, bla*.ext, bla*bla.ext *.ext, bla.*, bla.ext*, bla.*ext, bla.ex*t, ...
          */
          if (stars.size()==1)
          {
               int firstStar = ((Integer)stars.get(0)).intValue();
               return (name.toUpperCase().startsWith(filterString.substring(0,firstStar).toUpperCase()) && name.toUpperCase().endsWith(filterString.substring(firstStar+1).toUpperCase()));
          }

           /* if there are two or more stars, we have a filter like
            *bla*, *bla.*, *bla*.ext, bla*.ex*t, bla*bla*.ext *.*ext, bla*.*, bla.e*x*t,
            *bla*.ext, *bla*bla*, *.bla*bla*.ex*t, ...
          */
           if (stars.size()>1)
           {
                boolean isOk = true;
                int posInName = 0;
                int lastStar = 0;
                int nextStar = ((Integer)stars.get(0)).intValue();
                if (!name.toUpperCase().startsWith(filterString.substring(lastStar, nextStar).toUpperCase()))        
                {//check if name starts with the same as the filter
                    return false;
                }  
                lastStar = nextStar+1;      
                for(int i = 1; i<stars.size();i++)
                {//now we're checking the parts between two stars
                    nextStar = ((Integer)stars.get(i)).intValue();
                    posInName = name.toUpperCase().indexOf(filterString.substring(lastStar, nextStar).toUpperCase() , posInName);
                    if (posInName<0)
                    {
                        return false;
                    }
                    else
                    {
                        posInName += nextStar-lastStar;
                        lastStar = nextStar+1;
                    }
                }
                //and finally we have to check if the ending of the name ist the same of the ending of the filter
                return name.toUpperCase().endsWith(filterString.substring(lastStar).toUpperCase());
                
           }
        }
        return false;
    }
}
