package freaklamarsch.tools;

import java.awt.*;
import freaklamarsch.util.*;
import freaklamarsch.lang.*;
import java.lang.reflect.*;
import java.io.*;

/**
* <code>Thread</code> that runs an java application either by invoking it's main method or using <code>Runtime.exec</code>
* @author FReAK La Marsch
*/
public class FLMClassRunner extends Thread
{
    Class classToRun = null;
    String classpath = null;
    String classLocation = null;
    Process p;
    
    public FLMClassRunner(String cl, String cp)
    {
        this(null, cl, cp);
    }
    
    public FLMClassRunner(Frame parent, String cl, String cp)
    {
        if (FLMSystemUtil.canExec())      
        {
            String className = FLMClassLoader.calculateClassName(cl, cp);
            this.classLocation = className;
            this.classpath = cp;
        }
        else
        {
            FLMClassLoader classLoader = new FLMClassLoader(parent, cp);
            try
            {
                this.classToRun = classLoader.loadClass(cl);
            }
            catch(ClassNotFoundException cnfe){/*this should never happen...*/}
            catch(NoClassDefFoundError ncdfe){/*this means, that a needed class couldn't be found*/}
        }
    }

    public void run()
    {
        System.out.println("******************************\n********program   started********\n******************************");
        if (classToRun == null && classLocation != null)
        {//this is called when the System supports Runtime.exec()
            try
            {
                p = Runtime.getRuntime().exec(System.getProperty("java.home")+File.separator+"bin"+File.separator+"java -classpath \""+ classpath +"\" " + classLocation);
                BufferedInputStream inStream = new BufferedInputStream(p.getInputStream());
                BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
                String message;
                while(((message = in.readLine()) != null))
                {
                    System.out.println(message);
                }
            }
            catch(Exception e)
            {
                System.err.println("Exception while trying to run application: "+e);
            }
        }
        else if(classToRun != null)
        {
            String name = classToRun.getName();
            try
            {
                classToRun.getMethod("main", new Class[]{String[].class}).invoke(null, new String[1]);
            }
            catch(NoSuchMethodException ex)
            {
                System.err.println("Can't run the program!\nClass "+name+" does not define public static void main(String[]).");
            }
            catch(InvocationTargetException ex)
            {
                System.err.println("Exception while executing "+name+":\n"+ex.getTargetException());
            }
            catch(IllegalAccessException ex)
            {
                System.err.println("main(String[]) in class "+name+" is not public./nThis caused"+ex);
            }
        }
        else
        {//this should be the case if there where problems when loading the class
            System.out.println("Can't run that program!\nPlease check you classpath\nand try again.");
        }
        kill();
    }

    public void kill()
    {
        if (p!=null)
        {
            p.destroy();
            p = null;
        }
        classToRun = null;
        classpath = null;
        classLocation = null;
        System.out.println("******************************\n********program  stopped********\n******************************");
        stop();
    }
}